/* 
 *	Copyright (C) Chia-chen Kuo - April 2001
 *
 *  This file is part of DVD2AVIMFC, a free MPEG-2 decoder
 *	
 *  DVD2AVIMFC is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  DVD2AVIMFC is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 *  Modified by jsoto (idea taken from www.dvd-replica.com) May 2004
 *  1. Change the package name to Preview (for Previewing) to stress
 *     the differences in functionalities. All original capabilities
 *     of DVD2AVIMFC are hidden except the direct draw and 
 *     Mpeg2 decoder engine. Preview should be launched using CLI:
 *     Preview -AIF=[Name of the starting VOB file to display]
 *
 *  2. Adding a WM_USER message handling for positioning in the VOBs
 *     1) wParam=0 
 *     2) lParam=Sector (lba) starting from VTS_XX_1.VOB or VTS_XX_0.VOB 
 *
 *     Note that this interface has no idea about the internal structure
 *     of a DVD disc. It can display only a segment of a VOB file based
 *     upon the starting and ending sector. It is the responsibility of
 *     the caller program to send the correct WM_USER messages to
 *     it in order to work properly.
 *     When start playing, preview starts from the initial Cell sector, and goes 
 *     on following the VOB sectors order until the last sector. This could not
 *     be the right playing sequence after a while, but Preview does not know it.
 */

#define GLOBAL
extern "C"
{
#include "global.h"
#include "AC3Dec\ac3.h"
}

static char Version[] = "1.77.3dg1.1.0";

// If you define this, overlays will never be used.
//#define NO_OVERLAY_USE 1


// jsoto_start
#define TRACK_HEIGHT	30
//#define TRACK_HEIGHT	30
// jsoto_end
#define INIT_WIDTH		720
#define INIT_HEIGHT		480
#define MIN_WIDTH		160
#define MIN_HEIGHT		16

#define MASKCOLOR		RGB(0, 6, 0)

#define SAVE_AVI		1
#define	SAVE_D2V		2
#define SAVE_WAV		3
#define	OPEN_D2V		4
#define OPEN_VOB		5
#define OPEN_WAV		6
#define SAVE_BMP		7

// jsoto_start
// jsoto_end

#define PRIORITY_HIGH		1
#define PRIORITY_NORMAL		2
#define PRIORITY_LOW		3

#define LANGUAGE_DEFAULT	0
#define LANGUAGE_CHINESE	1
#define LANGUAGE_ENGLISH	2
#define LANGUAGE_JAPANESE	3

bool PopFileDlg(PTSTR, HWND, int);
ATOM MyRegisterClass(HINSTANCE);
LRESULT CALLBACK DVD2AVIWndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK Info(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK VideoList(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK AudioList(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK Delay(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK ClipResize(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK Luminance(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK Normalization(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK About(HWND, UINT, WPARAM, LPARAM);
static DWORD DDColorMatch(LPDIRECTDRAWSURFACE, COLORREF);
static void ShowInfo(bool);
static void ClearTrack(void);
static void CheckFlag(void);
static void Recovery(void);
void RefreshWindow(int);
static void SaveBMP(void);
static void OpenVideoFile(HWND);
static void OpenAudioFile(HWND);
DWORD WINAPI ProcessWAV(LPVOID n);
static void InitializeDVD2AVI(HINSTANCE hInstance, UINT Show);

HWND InitMain(HWND TheHWnd, int PosX, int PosY, UINT ShowWindow);
extern "C" int CumulStartSectors[MAX_FILE_NUMBER];
int	LastFileSector = 0;
extern "C" int SectorHit;

int FloatingInfo = 1;// Is the info window "floating"? (moves with the video window).
int InfoXPos = -1;	// Position of info panel.
int InfoYPos = -1;	// Position of info panel.


HWND ParenthWnd = 0;
UINT WindowStyle = WS_CHILD;	// The style of the video window.


static void StartupEnables(void);
static void FileLoadedEnables(void);
static void RunningEnables(void);

static int INIT_X, INIT_Y, Priority_Flag, Language_Flag, Edge_Width, Edge_Height;

static FILE *INIFile;
static char szPath[_MAX_PATH], szTemp[_MAX_PATH], szWindowClass[_MAX_PATH];

static HINSTANCE hInst;
static HANDLE hProcess;
HANDLE hThread;
static HWND hLeftButton, hLeftArrow, hRightArrow, hRightButton;

static DWORD threadId;
static RECT wrect, crect;
static int SoundDelay[MAX_FILE_NUMBER];
static char Outfilename[MAX_FILE_NUMBER][_MAX_PATH];

int	DisplayVideoOnly = 0;		// If true, there's no slider, and no buttons!

char *ExitOnEnd=NULL;
int CLIActive = 0;
int AutoIncrement = 0;
extern "C" float XDownSampFactor;			// X Image downsampling factor for bitmap file saving.
extern "C" float YDownSampFactor;			// Y Image downsampling factor for bitmap file saving.
extern "C" float YUVScaleX;					// X scaling factor for YUV display. YUVScaleX > 1 means enlarging.
extern "C" float YUVScaleY;					// Y scaling factor for YUV display.
extern "C" int	 AutoScale;		// 1 indicates YUVScaleY is calculated from YUVScaleX and the aspect ratio.

char BmpFileName[256];						// File name for bitmap file saving.



#include <objbase.h>
#include <crtdbg.h>		// This is needed for _ASSERTE
#include <stdio.h>
#include <stdarg.h>		// for va_start and va_end
#include <winuser.h>

#define Error(A) printf("ERROR: %s\n",A)


void PreviewSetVideoPosition( int PositionX, int PositionY)
{
	INIT_X = PositionX;
	INIT_Y = PositionY;
}

int PreviewLoadFile(char* FileName, int LoadMany = 1)
{
	int i=0;
	char aFName[_MAX_PATH];
	char suffix[_MAX_PATH];
	char *p, *q;
	int val;
	if(FileName != 0x00)
	{
		int tmp;
		char cwd[1024];

		File_Limit = 0;
		{
			strcpy(aFName, FileName);
			while (true)
			{
#if 0
				// I'm removing that stuff, it doesn't make much sense to do that...
				/* If the specified file does not include a path, use the
				   current directory. */
				if (!strstr(aFName, "\\"))
				{
					GetCurrentDirectory(sizeof(cwd) - 1, cwd);
					strcat(cwd, "\\");
					strcat(cwd, aFName);
				}
				else
				{
					strcpy(cwd, aFName);
				}
#else
				strcpy(cwd, aFName);
#endif
				if ((tmp = _open(cwd, _O_RDONLY | _O_BINARY | _O_SEQUENTIAL)) == -1) break;
				strcpy(Infilename[File_Limit], cwd);
				Infile[File_Limit] = tmp;
				File_Limit++;
				// LoadMany = FALSE tells us we just want this file.
				if(!LoadMany)
					break;
				// This part of the code will look for other parts of the current VTS and open the files.
				// For example, if FileName is VTS_01_1.VOB, it will try to open VTS_01_2.VOB, VTS_01_3.VOB
				// etc... This is great for viewing/processing a whole set of VOB files.
				// First scan back from the end of the name for an _ character.
				p = aFName+strlen(aFName);
				while (*p != '_' && p >= aFName) p--;
				if (*p != '_') break;
				// Now pick up the number value and increment it.
				p++;
				// JEANL: I'm not sure why you'd want to open VTS_01_1.VOB after VTS_01_0.VOB since they're
				// not even in the same domain... Use to be if (*p < '0' || *p > '9') break;
				if (*p < '1' || *p > '9') break;
				sscanf(p, "%d", &val);
				val++;
				// Save the suffix after the number.
				q = p;
				while (*p >= '0' && *p <= '9') p++;
				strcpy(suffix, p);
				// Write the new incremented number.
				sprintf(q, "%d", val);
				// Append the saved suffix.
				strcat(aFName, suffix);
			}
		}
	}
    Recovery();

	// Normalization not implemented

	RefreshWindow(true);

    if (File_Limit)
    {
		// Start a LOCATE_INIT thread. When it kills itself, it will start a
		// LOCATE_RIP thread by sending a WM_USER message to the main window.
		process.rightfile = File_Limit-1;
		process.rightlba = (int)(process.length[File_Limit-1]/BUFFER_SIZE);
		process.end = process.total - BUFFER_SIZE;
		process.endfile = File_Limit - 1;
		process.endloc = (process.length[File_Limit-1]/BUFFER_SIZE - 1)*BUFFER_SIZE;
		process.locate = LOCATE_INIT;
		if (!threadId || WaitForSingleObject(hThread, INFINITE)==WAIT_OBJECT_0)
		  hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
 		LastFileSector = (int)(process.total/BUFFER_SIZE);
		if(ParenthWnd)
			SetWindowText(ParenthWnd,FileName);
   }
	return File_Limit == 0;
}

void PreviewWaitForThread()
{
	if(hThread == NULL)
		return;
	while (WaitForSingleObject(hThread, 0)!=WAIT_OBJECT_0)
	{
		MSG msg;
		if (PeekMessage (&msg, hWnd, NULL, 0, 0)) 
		{
#if 0
			HACCEL hAccel = 0;
			if (!TranslateAccelerator(hWnd, hAccel, &msg))
			{
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
#else
#endif
		}
	}
	return;
}


int PreviewSaveToBMPFile(char* VideoFileName, int Sector, float Xfactor, float Yfactor, char* BMPFileName)
{

	// Get input arguments.
	XDownSampFactor = Xfactor;
	YDownSampFactor = Yfactor;
	strcpy(BmpFileName,BMPFileName);

	printf("Extracting bitmap \n\tat sector %d \n\tfrom file %s, \n\tXratio %f, \n\tYratio %f, \n\tinto bmp file %s\n",
		Sector,VideoFileName,XDownSampFactor, YDownSampFactor, BmpFileName);
	
	// Can't support enlarging the image!
	if(XDownSampFactor < 1) XDownSampFactor = 1;
	if(YDownSampFactor < 1) YDownSampFactor = 1;
	// Load file.
	PreviewLoadFile(VideoFileName,0);
	// We don't want a display here!
	Display_Flag = 0;

	// File_Limit is the number of files we opened! Hopefully just one! ;)
	if(File_Limit == 0)
	{
		// Error when file was opened... Let's exit...
		char Tmp[256];
		sprintf(Tmp,"Can't open file: %s",VideoFileName);
		Error(Tmp);
		return -1;
	}

	FILE* fd = fopen(BmpFileName,"wb");
	if(fd == NULL)
	{
		// Error can't open bmp file...
		char Tmp[256];
		sprintf(Tmp,"Can't open bmp file for writing: %s",BmpFileName);
		Error(Tmp);
		return -1;
	}
	fclose(fd);

	// Now, seek to the sector pass as an argument, and save the corresponding bitmap.
	// Wait for the thread started by PreviewLoadFile() to finish.
	PreviewWaitForThread();
	// Prepare a LOCATE_SCROLL job.
	if(process.length[File_Limit-1]/BUFFER_SIZE <= Sector)
	{
		// The sector is past the end of the file!
		char Tmp[256];
		sprintf(Tmp,"Sector %d past the end of the file",Sector);
		Error(Tmp);
		sprintf(Tmp,"End sector: %d\n",process.length[File_Limit-1]/BUFFER_SIZE);
		Error(Tmp);
		return -1;;
	}

	// Just call the handler...
	Display_Flag = 0;
	DVD2AVIWndProc(hWnd, WM_USER, ID_USERPOSITION, Sector);
	// Wait for the seek to finish.
	PreviewWaitForThread();
	// Fault_Flag would indicate a bad seek.
	if(Fault_Flag)
	{
		char Tmp[256];
		sprintf(Tmp,"Couldn't seek. Sector %d probably past the end of the file",Sector);
		Error(Tmp);
		return -1;;
	}

	// Now save bitmap!
	Display_Flag = 0;
	DVD2AVIWndProc(hWnd, WM_COMMAND, IDM_BMP, 0);
	PreviewWaitForThread();

	// Now cleanup and exit!!
	while (File_Limit)
	{
		File_Limit--;
		_close(Infile[File_Limit]);
	}
	Recovery();
	for (int i=0; i<8; i++)
		_aligned_free(block[i]);
	for (i=0; i<MAX_FILE_NUMBER; i++)
		free(Infilename[i]);
//	if (hLibrary)
//		FreeLibrary(hLibrary);
	ReleaseDC(hWnd, hDC);
	return 0;
}


// Main initialization function. Should be called with a handle to the parent window, the x and y
// for the video window, and SW_SHOW or SW_HIDE. Returns a handle to the video window.
HWND InitMain(HWND TheHWnd, int PosX, int PosY, UINT ShowWindow)
{
	ParenthWnd = TheHWnd;

	INIT_X = PosX;
	INIT_Y = PosY;

	// Initialize global strings
	HINSTANCE hInstance = GetModuleHandle(NULL);
	hInst = hInstance;
	strcpy(BmpFileName,"");
	strcpy(szWindowClass,"DVD2AVIMFC");
	MyRegisterClass(hInst);
	hWnd = CreateWindow(szWindowClass, "DVD2AVIMFC", WindowStyle,
		CW_USEDEFAULT, 0, INIT_WIDTH, INIT_HEIGHT, TheHWnd, NULL, hInstance, NULL);
	InitializeDVD2AVI(hInst,ShowWindow);
	return hWnd;
}


void InitializeDVD2AVI(HINSTANCE hInstance, UINT Show)
{
	int i=0;

	iDCT_Flag = IDCT_MMX;
	Scale_Flag = true;
	FO_Flag = FO_NONE;
	Method_Flag = AUDIO_DEMUXALL;
	Track_Flag = TRACK_1;
	DRC_Flag = DRC_NORMAL;
	DSDown_Flag = false;
	SRC_Flag = SRC_NONE;
	Norm_Ratio = 100;
	Priority_Flag = PRIORITY_NORMAL;
	Language_Flag = LANGUAGE_DEFAULT;

	Fps_Playback=true;

	// Test CPU
	__asm
	{
		mov			eax, 1
		cpuid
		test		edx, 0x00800000		// STD MMX
		jz			TEST_SSE
		mov			[cpu.mmx], 1
TEST_SSE:
		test		edx, 0x02000000		// STD SSE
		jz			TEST_SSE2
		mov			[cpu.ssemmx], 1
		mov			[cpu.ssefpu], 1
TEST_SSE2:
		test		edx, 0x04000000		// SSE2	
		jz			TEST_3DNOW
		mov			[cpu.sse2], 1
TEST_3DNOW:
		mov			eax, 0x80000001
		cpuid
		test		edx, 0x80000000		// 3D NOW
		jz			TEST_SSEMMX
		mov			[cpu._3dnow], 1
TEST_SSEMMX:
		test		edx, 0x00400000		// SSE MMX
		jz			TEST_END
		mov			[cpu.ssemmx], 1
TEST_END:
	}

	if (cpu.sse2)
	{
		CheckMenuItem(hMenu, IDM_SSE2, MF_CHECKED);

		DeleteMenu(hMenu, IDM_IDCT_MMX, 0);
		DeleteMenu(hMenu, IDM_IDCT_SSEMMX, 0);
	}
	else if (cpu.ssemmx)
	{
		DeleteMenu(hMenu, IDM_IDCT_MMX, 0);
		DeleteMenu(hMenu, IDM_IDCT_SSE2MMX, 0);
	}
	else
	{
		DeleteMenu(hMenu, IDM_IDCT_SSEMMX, 0);
		DeleteMenu(hMenu, IDM_IDCT_SSE2MMX, 0);
	}

	if (cpu.mmx)
		CheckMenuItem(hMenu, IDM_MMX, MF_CHECKED);
	else
		DestroyWindow(hWnd);

	if (cpu.ssemmx)
		CheckMenuItem(hMenu, IDM_SSEMMX, MF_CHECKED);

	if (cpu._3dnow)
		CheckMenuItem(hMenu, IDM_3DNOW, MF_CHECKED);

	if (cpu.ssefpu)
		CheckMenuItem(hMenu, IDM_SSEFPU, MF_CHECKED);

	// Create control
	UINT ShowIt = WS_VISIBLE;
	if(DisplayVideoOnly)
		ShowIt = 0;

	int InitWidth = (int)(INIT_WIDTH * YUVScaleX);
	int InitHeight = (int)(INIT_HEIGHT * YUVScaleY);
	hTrack = CreateWindow(TRACKBAR_CLASS, NULL,
		WS_CHILD | ShowIt | WS_DISABLED | TBS_ENABLESELRANGE | TBS_NOTICKS | TBS_TOP,
		0, InitHeight, InitWidth-4*TRACK_HEIGHT, TRACK_HEIGHT, hWnd, (HMENU) ID_TRACKBAR, hInst, NULL);

	SendMessage(hTrack, TBM_SETRANGE, (WPARAM) true, (LPARAM) MAKELONG(0, TRACK_PITCH));

	hLeftButton = CreateWindow("BUTTON", "[",
		WS_CHILD | ShowIt | WS_DLGFRAME | WS_DISABLED,
		InitWidth-4*TRACK_HEIGHT, InitHeight,
		TRACK_HEIGHT, TRACK_HEIGHT, hWnd, (HMENU) ID_LEFT_BUTTON, hInst, NULL);

	hLeftArrow = CreateWindow("BUTTON", "<",
		WS_CHILD | ShowIt | WS_DLGFRAME | WS_DISABLED,
		InitWidth-3*TRACK_HEIGHT, InitHeight,
		TRACK_HEIGHT, TRACK_HEIGHT, hWnd, (HMENU) ID_LEFT_ARROW, hInst, NULL);

	hRightArrow = CreateWindow("BUTTON", ">",
		WS_CHILD | ShowIt | WS_DLGFRAME | WS_DISABLED,
		InitWidth-2*TRACK_HEIGHT, InitHeight,
		TRACK_HEIGHT, TRACK_HEIGHT, hWnd, (HMENU) ID_RIGHT_ARROW, hInst, NULL);

	hRightButton = CreateWindow("BUTTON", "]",
		WS_CHILD | ShowIt | WS_DLGFRAME | WS_DISABLED,
		InitWidth-TRACK_HEIGHT, InitHeight,
		TRACK_HEIGHT, TRACK_HEIGHT, hWnd, (HMENU) ID_RIGHT_BUTTON, hInst, NULL);

	Coded_Picture_Width = INIT_WIDTH;
	Coded_Picture_Height = INIT_HEIGHT;

	ResizeWindow(INIT_WIDTH, INIT_HEIGHT);
//	MoveWindow(hWnd, INIT_X, INIT_Y, InitWidth+Edge_Width, InitHeight+Edge_Height+TRACK_HEIGHT, true);
	ShowWindow(hWnd, Show);

//jsoto_start
	SetWindowText(hWnd, "VobBlanker Preview");
//jsoto_end


	StartupEnables();

	CheckFlag();

	D2V_176_Style = false;

	static HINSTANCE hLibrary;
	PreScale_Ratio = 1.0;

	hDC = GetDC(hWnd);
	hMenu = GetMenu(hWnd);
	hProcess = GetCurrentProcess();

	for (i=0; i<MAX_FILE_NUMBER; i++)
		Infilename[i] = (char*)malloc(_MAX_PATH);

	for (i=0; i<8; i++)
		block[i] = (short *)_aligned_malloc(sizeof(short)*64, 64);

	Initialize_REF_IDCT();
	Initialize_FPU_IDCT();

	// register VFAPI
	HKEY key; DWORD trash;

	if (RegCreateKeyEx(HKEY_CURRENT_USER, "Software\\VFPlugin", 0, "",
		REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &key, &trash) == ERROR_SUCCESS)
	{
		if (_getcwd(szBuffer, _MAX_PATH)!=NULL)
		{
			if (szBuffer[strlen(szBuffer)-1] != '\\')
				strcat(szBuffer, "\\");

			strcpy(szPath, szBuffer);

			struct _finddata_t vfpfile;
			if (_findfirst("DVD2AVIMFCdg.vfp", &vfpfile) != -1L)
			{
				strcat(szBuffer, "DVD2AVIMFCdg.vfp");

				RegSetValueEx(key, "DVD2AVIMFC", 0, REG_SZ, (LPBYTE)szBuffer, strlen(szBuffer));
				CheckMenuItem(hMenu, IDM_VFAPI, MF_CHECKED);
			}

			RegCloseKey(key);
		}
	}

	// load Library
	OpenDVD_Flag = true;

	if ((hLibrary = LoadLibrary("OpenDVD.dll")) == NULL)
		OpenDVD_Flag = false;
	else
	{
		if ((KeyOp = (pfnKeyOp) GetProcAddress(hLibrary, "KeyOp")) == NULL)
			OpenDVD_Flag = false;

		if ((BufferOp = (pfnBufferOp) GetProcAddress(hLibrary, "BufferOp")) == NULL)
			OpenDVD_Flag = false;
	}

	if (!OpenDVD_Flag)
		DeleteMenu(GetSubMenu(hMenu,4), 3, MF_BYPOSITION);

}

void SetVideoScalingFactor(float XScale, float YScale, int autoscale)
{
	AutoScale = autoscale;
	YUVScaleX = max(XScale,0.01f);
	YUVScaleX = min(YUVScaleX,MAX_VIDEO_SCALE);
	YUVScaleY = max(YScale,0.01f);
	YUVScaleY = min(YUVScaleY,MAX_VIDEO_SCALE);
}

// Processes messages for the main window
LRESULT CALLBACK DVD2AVIWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	DWORD wmId, wmEvent;

	HDC hdc;
	PAINTSTRUCT ps;
	static HINSTANCE hLibrary;

	int i, j, tmp;

	switch (message)
	{
		case WM_COMMAND:
			wmId    = LOWORD(wParam);
			wmEvent = HIWORD(wParam);

			// parse the menu selections
			switch (wmId)
			{
				case IDM_OPEN_AUTO:
					AutoIncrement = 1;
					DialogBox(hInst, (LPCTSTR)IDD_FILELIST, hWnd, (DLGPROC)VideoList);
					break;

				case IDM_OPEN:
					AutoIncrement = 0;
					DialogBox(hInst, (LPCTSTR)IDD_FILELIST, hWnd, (DLGPROC)VideoList);
					break;

				case IDM_PREVIEW:
				case IDM_PLAY:
					if (IsWindowEnabled(hTrack))
					{
						RunningEnables();
						Display_Flag = true;
// jsoto: Comment next line
//						ShowInfo(true);

						if (wmId == IDM_PREVIEW)
							process.locate = LOCATE_RIP;
						else
							process.locate = LOCATE_PLAY;

						if (WaitForSingleObject(hThread, INFINITE)==WAIT_OBJECT_0)
							hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
					}
					break;

				case IDM_SAVE:
					MessageBox(hWnd, "Not implemented in this version. See readme file.", NULL, MB_OK | MB_ICONERROR);
#if 0
					if (PopFileDlg(szOutput, hWnd, SAVE_AVI))
					{
						AVI_Flag = true;
						Display_Flag = false;
						RunningEnables();
// jsoto: Comment next line
//						ShowInfo(true);

						process.locate = LOCATE_RIP;

						if (WaitForSingleObject(hThread, INFINITE)==WAIT_OBJECT_0)
							hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
					}
#endif
					break;

				case IDM_PROCESS_WAV:
					DialogBox(hInst, (LPCTSTR)IDD_FILELIST, hWnd, (DLGPROC)AudioList);
					break;

				case IDM_SAVE_D2V_176:
					D2V_176_Style = true;
					goto saved2v;

				case IDM_SAVE_D2V:
					D2V_176_Style = false;
saved2v:
					if (CLIActive || PopFileDlg(szOutput, hWnd, SAVE_D2V))
					{
						sprintf(szBuffer, "%s.d2v", szOutput);

						D2VFile = fopen(szBuffer, "w+");

						if (D2VFile != 0)
						{
							D2V_Flag = true;
							Display_Flag = false;
							RunningEnables();
// jsoto: Comment next line
//							ShowInfo(true);

							process.locate = LOCATE_RIP;
							if (CLIActive || WaitForSingleObject(hThread, INFINITE)==WAIT_OBJECT_0)
							{
								CLIActive = 0;
								hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
							}
						}
					}
					break;

				case IDM_LOAD_D2V:
					if (PopFileDlg(szInput, hWnd, OPEN_D2V))
					{
//D2V_PROCESS:
						long pos;
						char line[255];
						int D2v176 = 0;

						D2VFile = fopen(szInput, "r");

						while (File_Limit)
						{
							File_Limit--;
							_close(Infile[File_Limit]);
						}

						fscanf(D2VFile, "DVD2AVIProjectFile\n%d\n", &File_Limit);

						i = File_Limit;
						while (i)
						{
							fscanf(D2VFile, "%d ", &j);
							fgets(Infilename[File_Limit-i], j+1, D2VFile);
							if ((Infile[File_Limit-i] = _open(Infilename[File_Limit-i], _O_RDONLY | _O_BINARY | _O_SEQUENTIAL))==-1)
							{
								while (i<File_Limit)
								{
									_close(Infile[File_Limit-i-1]);
									i++;
								}

								File_Limit = 0;
								break;
							}

							i--;
						}

						Recovery();

						fscanf(D2VFile, "\nStream_Type=%d,%X,%X\n", &SystemStream_Flag, &lfsr0, &lfsr1);
						if (lfsr0 || lfsr1)
							KeyOp_Flag = KEY_INPUT;
						else
							KeyOp_Flag = KEY_OFF;

						pos = ftell(D2VFile);
						fgets(line, sizeof(line) - 1, D2VFile);
						if (strstr(line, "MMX") == 0) D2v176 = 1;
						fseek(D2VFile, pos, SEEK_SET);
//						if (D2v176) OutputDebugString("DVD2AVI: 176\n");

						if (!D2v176)
						{
							fscanf(D2VFile, "iDCT_Algorithm=%d (1:MMX 2:SSEMMX 3:FPU 4:REF 5:SSE2MMX)\n", &iDCT_Flag);
							fscanf(D2VFile, "YUVRGB_Scale=%d (0:TVScale 1:PCScale)\n", &Scale_Flag);
							fscanf(D2VFile, "Luminance_Filter=%d,%d (Gamma, Offset)\n", &LumGamma, &LumOffset);
						}
						else
						{
							fscanf(D2VFile, "iDCT_Algorithm=%d\n", &iDCT_Flag);
							fscanf(D2VFile, "YUVRGB_Scale=%d\n", &Scale_Flag);
							fscanf(D2VFile, "Luminance=%d,%d\n", &LumGamma, &LumOffset);
							LumGamma -= 128;
						}

						if (LumGamma || LumOffset)
						{
							CheckMenuItem(hMenu, IDM_LUMINANCE, MF_CHECKED);
							Luminance_Flag = true;				
						}
						else
						{
							CheckMenuItem(hMenu, IDM_LUMINANCE, MF_UNCHECKED);
							Luminance_Flag = false;
						}

						if (!D2v176)
						{
							fscanf(D2VFile, "Picture_Size=%d,%d,%d,%d,%d,%d (ClipLeft, ClipRight, ClipTop, ClipBottom)\n", 
								&Clip_Left, &Clip_Right, &Clip_Top, &Clip_Bottom, &Squeeze_Width, &Squeeze_Height);
						}
						else
						{
							fscanf(D2VFile, "Picture_Size=%d,%d,%d,%d,%d,%d\n", 
								&Clip_Left, &Clip_Right, &Clip_Top, &Clip_Bottom, &Squeeze_Width, &Squeeze_Height);
						}

						if (Clip_Top || Clip_Bottom || Clip_Left || Clip_Right || Squeeze_Width || Squeeze_Height)
						{
							CheckMenuItem(hMenu, IDM_CLIPRESIZE, MF_CHECKED);
							ClipResize_Flag = true;				
						}
						else
						{
							CheckMenuItem(hMenu, IDM_CLIPRESIZE, MF_UNCHECKED);
							ClipResize_Flag = false;
						}

						if (!D2v176)
						{
							fscanf(D2VFile, "Field_Operation=%d (0:None 1:ForcedFILM 2:SwapOrder)\n", &FO_Flag);
						}
						else
						{
							fscanf(D2VFile, "Field_Operation=%d\n", &FO_Flag);
						}
						fscanf(D2VFile, "Frame_Rate=%d\n", &frame_rate);

						CheckFlag();

						if (File_Limit)
						{
							fscanf(D2VFile, "Location=%d,%X,%d,%X\n", &process.leftfile, 
								&process.leftlba, &process.rightfile, &process.rightlba);

							process.startfile = process.leftfile;
							process.startloc = process.leftlba * BUFFER_SIZE;
							process.endfile = process.rightfile;
							process.endloc = (process.rightlba - 1) * BUFFER_SIZE;

							process.run = 0;
							for (i=0; i<process.startfile; i++)
								process.run += process.length[i];
							process.start = process.run + process.startloc;

							process.end = 0;
							for (i=0; i<process.endfile; i++)
								process.end += process.length[i];
							process.end += process.endloc;

							process.trackleft = (int)(process.start*TRACK_PITCH/process.total);
							process.trackright = (int)(process.end*TRACK_PITCH/process.total);

							process.locate = LOCATE_INIT;

							if (!threadId || WaitForSingleObject(hThread, INFINITE)==WAIT_OBJECT_0)
								hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
						}
					}
					break;

				case IDM_STOP:
					Stop_Flag = true;
					FileLoadedEnables();

					if (Pause_Flag)
						ResumeThread(hThread);
					break;

				case IDM_STORE_RGB24:
					if (Store_Flag!=STORE_RGB24)
					{
						if (DDOverlay_Flag)
							IDirectDrawSurface_UpdateOverlay(lpOverlay, NULL, lpPrimary, NULL, DDOVER_HIDE, NULL);
						Store_Flag = STORE_RGB24;
						CheckMenuItem(hMenu, IDM_STORE_RGB24, MF_CHECKED);
						CheckMenuItem(hMenu, IDM_STORE_YUY2, MF_UNCHECKED);
						RefreshWindow(true);
					}
					break;

				case IDM_STORE_YUY2:
					if (Store_Flag!=STORE_YUY2)
					{
						Store_Flag = STORE_YUY2;
						CheckMenuItem(hMenu, IDM_STORE_RGB24, MF_UNCHECKED);
						CheckMenuItem(hMenu, IDM_STORE_YUY2, MF_CHECKED);

						if (DDOverlay_Flag)
						{
							GetClientRect(hWnd, &crect);
							crect.bottom -= TRACK_HEIGHT;
							
							HBRUSH hBrush = CreateSolidBrush(MASKCOLOR);
							FillRect(hDC, &crect, hBrush);
							DeleteObject(hBrush);

							crect.bottom += TRACK_HEIGHT;
						}

						RefreshWindow(true);
					}
					break;

				case IDM_AUDIO_NONE:
					Method_Flag = AUDIO_NONE;
					CheckMenuItem(hMenu, IDM_AUDIO_NONE, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_DEMUX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DEMUXALL, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DECODE, MF_UNCHECKED);
					break;

				case IDM_DEMUX:
					Method_Flag = AUDIO_DEMUX;
					CheckMenuItem(hMenu, IDM_AUDIO_NONE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DEMUX, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_DEMUXALL, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DECODE, MF_UNCHECKED);
					break;

				case IDM_DEMUXALL:
					Method_Flag = AUDIO_DEMUXALL;
					CheckMenuItem(hMenu, IDM_AUDIO_NONE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DEMUX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DEMUXALL, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_DECODE, MF_UNCHECKED);
					break;

				case IDM_DECODE:
					Method_Flag = AUDIO_DECODE;
					CheckMenuItem(hMenu, IDM_AUDIO_NONE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DEMUX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DEMUXALL, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DECODE, MF_CHECKED);
					break;

				case IDM_TRACK_1:
					ClearTrack();
					Track_Flag = TRACK_1;
					CheckMenuItem(hMenu, IDM_TRACK_1, MF_CHECKED);
					break;

				case IDM_TRACK_2:
					ClearTrack();
					Track_Flag = TRACK_2;
					CheckMenuItem(hMenu, IDM_TRACK_2, MF_CHECKED);
					break;

				case IDM_TRACK_3:
					ClearTrack();
					Track_Flag = TRACK_3;
					CheckMenuItem(hMenu, IDM_TRACK_3, MF_CHECKED);
					break;

				case IDM_TRACK_4:
					ClearTrack();
					Track_Flag = TRACK_4;
					CheckMenuItem(hMenu, IDM_TRACK_4, MF_CHECKED);
					break;

				case IDM_TRACK_5:
					ClearTrack();
					Track_Flag = TRACK_5;
					CheckMenuItem(hMenu, IDM_TRACK_5, MF_CHECKED);
					break;

				case IDM_TRACK_6:
					ClearTrack();
					Track_Flag = TRACK_6;
					CheckMenuItem(hMenu, IDM_TRACK_6, MF_CHECKED);
					break;

				case IDM_TRACK_7:
					ClearTrack();
					Track_Flag = TRACK_7;
					CheckMenuItem(hMenu, IDM_TRACK_7, MF_CHECKED);
					break;

				case IDM_TRACK_8:
					ClearTrack();
					Track_Flag = TRACK_8;
					CheckMenuItem(hMenu, IDM_TRACK_8, MF_CHECKED);
					break;

				case IDM_DRC_NONE:
					DRC_Flag = DRC_NONE;
					CheckMenuItem(hMenu, IDM_DRC_NONE, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_DRC_LIGHT, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DRC_NORMAL, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DRC_HEAVY, MF_UNCHECKED);
					break;

				case IDM_DRC_LIGHT:
					DRC_Flag = DRC_LIGHT;
					CheckMenuItem(hMenu, IDM_DRC_NONE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DRC_LIGHT, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_DRC_NORMAL, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DRC_HEAVY, MF_UNCHECKED);
					break;

				case IDM_DRC_NORMAL:
					DRC_Flag = DRC_NORMAL;
					CheckMenuItem(hMenu, IDM_DRC_NONE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DRC_LIGHT, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DRC_NORMAL, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_DRC_HEAVY, MF_UNCHECKED);
					break;

				case IDM_DRC_HEAVY:
					DRC_Flag = DRC_HEAVY;
					CheckMenuItem(hMenu, IDM_DRC_NONE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DRC_LIGHT, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DRC_NORMAL, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_DRC_HEAVY, MF_CHECKED);
					break;

				case IDM_DSDOWN:
					if (DSDown_Flag)
						CheckMenuItem(hMenu, IDM_DSDOWN, MF_UNCHECKED);
					else
						CheckMenuItem(hMenu, IDM_DSDOWN, MF_CHECKED);

					DSDown_Flag = !DSDown_Flag;
					break;

				case IDM_PRESCALE:
					if (PreScale_Ratio==1.0 && Check_Flag && Method_Flag!=AUDIO_DEMUXALL && IsWindowEnabled(hTrack))
					{
						Decision_Flag = true;
						Display_Flag = false;
						RunningEnables();
						ShowInfo(true);

						process.locate = LOCATE_RIP;
						PreScale_Ratio = 1.0;

						if (WaitForSingleObject(hThread, INFINITE)==WAIT_OBJECT_0)
							hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
					}
					else
					{
						CheckMenuItem(hMenu, IDM_PRESCALE, MF_UNCHECKED);
						PreScale_Ratio = 1.0;
					}
					break;

				case IDM_IDCT_MMX:
					iDCT_Flag = IDCT_MMX;
					CheckMenuItem(hMenu, IDM_IDCT_MMX, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_SSEMMX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_SSE2MMX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_FPU, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_REF, MF_UNCHECKED);
					break;

				case IDM_IDCT_SSEMMX:
					iDCT_Flag = IDCT_SSEMMX;
					CheckMenuItem(hMenu, IDM_IDCT_MMX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_SSEMMX, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_SSE2MMX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_FPU, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_REF, MF_UNCHECKED);
					break;

				case IDM_IDCT_SSE2MMX:
					iDCT_Flag = IDCT_SSE2MMX;
					CheckMenuItem(hMenu, IDM_IDCT_MMX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_SSEMMX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_SSE2MMX, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_FPU, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_REF, MF_UNCHECKED);
					break;

				case IDM_IDCT_FPU:
					iDCT_Flag = IDCT_FPU;
					CheckMenuItem(hMenu, IDM_IDCT_MMX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_SSEMMX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_SSE2MMX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_FPU, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_REF, MF_UNCHECKED);
					break;

				case IDM_IDCT_REF:
					iDCT_Flag = IDCT_REF;
					CheckMenuItem(hMenu, IDM_IDCT_MMX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_SSEMMX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_SSE2MMX, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_FPU, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_IDCT_REF, MF_CHECKED);
					break;

				case IDM_FO_NONE:
					FO_Flag = FO_NONE;
					CheckMenuItem(hMenu, IDM_FO_NONE, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_FO_FILM, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_FO_SWAP, MF_UNCHECKED);
					SetDlgItemText(hDlg, IDC_INFO, "");
					break;

				case IDM_FO_FILM:
					FO_Flag = FO_FILM;
					CheckMenuItem(hMenu, IDM_FO_NONE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_FO_FILM, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_FO_SWAP, MF_UNCHECKED);
					SetDlgItemText(hDlg, IDC_INFO, "");
					break;

				case IDM_FO_SWAP:
					FO_Flag = FO_SWAP;
					CheckMenuItem(hMenu, IDM_FO_NONE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_FO_FILM, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_FO_SWAP, MF_CHECKED);
					SetDlgItemText(hDlg, IDC_INFO, "");
					break;

				case IDM_TVSCALE:
					RGB_Scale = 0x1000200010002000;
					RGB_Offset = 0x0000000000000000;
					RGB_CBU = 0x000038B4000038B4;
					RGB_CGX = 0xF4FDE926F4FDE926;
					RGB_CRV = 0x00002CDD00002CDD;

					Scale_Flag = false;
					CheckMenuItem(hMenu, IDM_TVSCALE, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_PCSCALE, MF_UNCHECKED);

					RefreshWindow(true);
					break;

				case IDM_PCSCALE:
					RGB_Scale = 0x1000254310002543;
					RGB_Offset = 0x0010001000100010;
					RGB_CBU = 0x0000408D0000408D;
					RGB_CGX = 0xF377E5FCF377E5FC;
					RGB_CRV = 0x0000331300003313;

					Scale_Flag = true;
					CheckMenuItem(hMenu, IDM_TVSCALE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_PCSCALE, MF_CHECKED);

					RefreshWindow(true);
					break;

				case IDM_CLIPRESIZE:
					DialogBox(hInst, (LPCTSTR)IDD_CLIPRESIZE, hWnd, (DLGPROC)ClipResize);
					break;

				case IDM_LUMINANCE:
					DialogBox(hInst, (LPCTSTR)IDD_LUMINANCE, hWnd, (DLGPROC)Luminance);
					break;

				case IDM_NORM:
					DialogBox(hInst, (LPCTSTR)IDD_NORM, hWnd, (DLGPROC)Normalization);
					break;

				case IDM_PP_HIGH:
					Priority_Flag = PRIORITY_HIGH;
					SetPriorityClass(hProcess, HIGH_PRIORITY_CLASS);
					CheckMenuItem(hMenu, IDM_PP_HIGH, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_PP_NORMAL, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_PP_LOW, MF_UNCHECKED);
					break;

				case IDM_PP_NORMAL:
					Priority_Flag = PRIORITY_NORMAL;
					SetPriorityClass(hProcess, NORMAL_PRIORITY_CLASS);
					CheckMenuItem(hMenu, IDM_PP_HIGH, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_PP_NORMAL, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_PP_LOW, MF_UNCHECKED);
					break;

				case IDM_PP_LOW:
					Priority_Flag = PRIORITY_LOW;
					SetPriorityClass(hProcess, IDLE_PRIORITY_CLASS);
					CheckMenuItem(hMenu, IDM_PP_HIGH, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_PP_NORMAL, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_PP_LOW, MF_CHECKED);
					break;

				case IDM_LANG_DEFAULT:
					Language_Flag = LANGUAGE_DEFAULT;
					CheckMenuItem(hMenu, IDM_LANG_DEFAULT, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_LANG_CHINESE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_LANG_ENGLISH, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_LANG_JAPANESE, MF_UNCHECKED);
					break;

				case IDM_LANG_CHINESE:
					Language_Flag = LANGUAGE_CHINESE;
					CheckMenuItem(hMenu, IDM_LANG_DEFAULT, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_LANG_CHINESE, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_LANG_ENGLISH, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_LANG_JAPANESE, MF_UNCHECKED);
					break;

				case IDM_LANG_ENGLISH:
					Language_Flag = LANGUAGE_ENGLISH;
					CheckMenuItem(hMenu, IDM_LANG_DEFAULT, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_LANG_CHINESE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_LANG_ENGLISH, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_LANG_JAPANESE, MF_UNCHECKED);
					break;

				case IDM_LANG_JAPANESE:
					Language_Flag = LANGUAGE_JAPANESE;
					CheckMenuItem(hMenu, IDM_LANG_DEFAULT, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_LANG_CHINESE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_LANG_ENGLISH, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_LANG_JAPANESE, MF_CHECKED);
					break;

				case IDM_PAUSE:
					if (Pause_Flag)
						ResumeThread(hThread);
					else
						SuspendThread(hThread);

					Pause_Flag = !Pause_Flag;
					break;

				case IDM_INFO:
// JEANL: Changing that to use lParam to determine whether to show info or not.
					Info_Flag = !lParam;
					if (Info_Flag)
					{
						DestroyWindow(hDlg);
						Info_Flag = false;
						hDlg = NULL;
					}
					else
						ShowInfo(true);
					break;

				case IDM_DISPLAY:
					Display_Flag = !Display_Flag;
					break;

				case IDM_BMP:
					tmp = Store_Flag;
					if (Store_Flag != STORE_RGB24)
					{
						if (DDOverlay_Flag)
							IDirectDrawSurface_UpdateOverlay(lpOverlay, NULL, lpPrimary, NULL, DDOVER_HIDE, NULL);
						Store_Flag = STORE_RGB24;
						RefreshWindow(true);
					}
					SaveBMP();
					if (tmp != STORE_RGB24)
					{
						Store_Flag = STORE_YUY2;
						if (DDOverlay_Flag && 1)
						{
							GetClientRect(hWnd, &crect);
							if(!DisplayVideoOnly)
								crect.bottom -= TRACK_HEIGHT;
							HBRUSH hBrush = CreateSolidBrush(MASKCOLOR);
							FillRect(hDC, &crect, hBrush);
							DeleteObject(hBrush);
							if(!DisplayVideoOnly)
								crect.bottom += TRACK_HEIGHT;
						}
						RefreshWindow(true);
					}
					break;

				case IDM_SRC_NONE:
					SRC_Flag = SRC_NONE;
					CheckMenuItem(hMenu, IDM_SRC_NONE, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_SRC_LOW, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_MID, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_HIGH, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_UHIGH, MF_UNCHECKED);
					break;

				case IDM_SRC_LOW:
					SRC_Flag = SRC_LOW;
					CheckMenuItem(hMenu, IDM_SRC_NONE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_LOW, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_SRC_MID, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_HIGH, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_UHIGH, MF_UNCHECKED);
					break;

				case IDM_SRC_MID:
					SRC_Flag = SRC_MID;
					CheckMenuItem(hMenu, IDM_SRC_NONE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_LOW, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_MID, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_SRC_HIGH, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_UHIGH, MF_UNCHECKED);
					break;

				case IDM_SRC_HIGH:
					SRC_Flag = SRC_HIGH;
					CheckMenuItem(hMenu, IDM_SRC_NONE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_LOW, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_MID, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_HIGH, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_SRC_UHIGH, MF_UNCHECKED);
					break;

				case IDM_SRC_UHIGH:
					SRC_Flag = SRC_UHIGH;
					CheckMenuItem(hMenu, IDM_SRC_NONE, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_LOW, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_MID, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_HIGH, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_SRC_UHIGH, MF_CHECKED);
					break;

				case IDM_KEY_OFF:
					KeyOp_Flag = KEY_OFF;
					CheckMenuItem(hMenu, IDM_KEY_OFF, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_KEY_INPUT, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_KEY_OP, MF_UNCHECKED);
					break;

				case IDM_KEY_INPUT:
					KeyOp_Flag = KEY_INPUT;
					CheckMenuItem(hMenu, IDM_KEY_OFF, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_KEY_INPUT, MF_CHECKED);
					CheckMenuItem(hMenu, IDM_KEY_OP, MF_UNCHECKED);
					break;

				case IDM_KEY_OP:
					KeyOp_Flag = KEY_OP;
					CheckMenuItem(hMenu, IDM_KEY_OFF, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_KEY_INPUT, MF_UNCHECKED);
					CheckMenuItem(hMenu, IDM_KEY_OP, MF_CHECKED);
					break;

				case IDM_ABOUT:
					DialogBox(hInst, (LPCTSTR)IDD_ABOUT, hWnd, (DLGPROC)About);
					break;

				case IDM_JACKEI:
					ShellExecute(NULL, "open", "http://arbor.ee.ntu.edu.tw/~jackei/dvd2avi/", NULL, NULL, SW_SHOWNORMAL);
					break;

				case IDM_NEURON2:
					ShellExecute(NULL, "open", "http://neuron2.net/fixd2v/decodefix.html", NULL, NULL, SW_SHOWNORMAL);
					break;

				case IDM_EXIT:
					DestroyWindow(hWnd);
					break;

				case ID_LEFT_BUTTON:
					if (IsWindowEnabled(hTrack))
					{
						SetFocus(hWnd);

						if ((process.file < process.rightfile) || (process.file==process.rightfile && process.lba<process.rightlba))
						{
							process.leftfile = process.file;
							process.leftlba = process.lba;

							process.run = 0;
							for (i=0; i<process.leftfile; i++)
								process.run += process.length[i];
							process.trackleft = (int)((process.run + (__int64)process.leftlba * BUFFER_SIZE) * TRACK_PITCH / process.total);
							SendMessage(hTrack, TBM_SETPOS, (WPARAM) true, process.trackleft);
							SendMessage(hTrack, TBM_SETSEL, (WPARAM) true, (LPARAM) MAKELONG(process.trackleft, process.trackright));
						}
					}
					break;

				case ID_LEFT_ARROW:
					SetFocus(hWnd);

					if (WaitForSingleObject(hThread, 0)==WAIT_OBJECT_0)
					{
						Display_Flag = true;

						process.locate = LOCATE_BACKWARD;
						hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
					}
					break;

				case ID_RIGHT_ARROW:
					SetFocus(hWnd);

					if (WaitForSingleObject(hThread, 0)==WAIT_OBJECT_0)
					{
						Display_Flag = true;

						process.locate = LOCATE_FORWARD;
						hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
					}
					break;

				case ID_RIGHT_BUTTON:
					if (IsWindowEnabled(hTrack))
					{
						SetFocus(hWnd);

						if ((process.file>process.leftfile) || (process.file==process.leftfile && process.lba>process.leftlba))
						{
							process.rightfile = process.file;
							process.rightlba = process.lba;

							process.run = 0;
							for (i=0; i<process.rightfile; i++)
								process.run += process.length[i];
							process.trackright = (int)((process.run + (__int64)process.rightlba*BUFFER_SIZE)*TRACK_PITCH/process.total);
							SendMessage(hTrack, TBM_SETPOS, (WPARAM) true, process.trackright);
							SendMessage(hTrack, TBM_SETSEL, (WPARAM) true, (LPARAM) MAKELONG(process.trackleft, process.trackright));
						}
					}
					break;

				default:
					return DefWindowProc(hWnd, message, wParam, lParam);
			}
			break;

		case WM_USER:
			// The CLI-invoked LOCATE_INIT thread is finished.
			// Kick off a LOCATE_RIP thread.
			if (wParam == 0xdeadbeef)
			{
//				goto saved2v;
			}
//jsoto_start
			if (wParam == ID_USERPOSITION)
			{
				if (WaitForSingleObject(hThread, 0)==WAIT_OBJECT_0)
				{
					process.startloc = process.start = ((__int64)lParam)*BUFFER_SIZE;

					process.startfile = 0; process.run = 0;
					while (process.startloc > process.length[process.startfile])
					{
						process.startloc -= process.length[process.startfile];
						process.run += process.length[process.startfile];
						process.startfile++;
					}

					process.end = process.total - BUFFER_SIZE;
					process.endfile = File_Limit - 1;
					process.endloc = (process.length[File_Limit-1]/BUFFER_SIZE-1)*BUFFER_SIZE;

					process.locate = LOCATE_SCROLL;

					hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
				}
			}
// INISECTOR 
			if (wParam == ID_USERINISECTOR || wParam == ID_USERENDSECTOR )
			{
				if (WaitForSingleObject(hThread, 0)==WAIT_OBJECT_0)
				{
					Display_Flag = true;

					process.startloc = process.start = ((__int64)lParam)*BUFFER_SIZE;

					process.startfile = 0; process.run = 0;
					while (process.startloc > process.length[process.startfile])
					{
						process.startloc -= process.length[process.startfile];
						process.run += process.length[process.startfile];
						process.startfile++;
					}

					process.end = process.total - BUFFER_SIZE;
					process.endfile = File_Limit - 1;
					process.endloc = (process.length[File_Limit-1]/BUFFER_SIZE-1)*BUFFER_SIZE;

					process.locate = LOCATE_SCROLL;

					hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);

				}
			}

//jsoto_end
			break;

		case WM_CREATE:
			break;


		case WM_HSCROLL:
			SetFocus(hWnd);

			if (WaitForSingleObject(hThread, 0)==WAIT_OBJECT_0)
			{
				int trackpos;

				Display_Flag = true;

				trackpos = SendMessage(hTrack, TBM_GETPOS, 0, 0);
				process.startloc = process.start = process.total*trackpos/TRACK_PITCH;

				process.startfile = 0; process.run = 0;
				while (process.startloc > process.length[process.startfile])
				{
					process.startloc -= process.length[process.startfile];
					process.run += process.length[process.startfile];
					process.startfile++;
				}

				process.end = process.total - BUFFER_SIZE;
				process.endfile = File_Limit - 1;
				process.endloc = (process.length[File_Limit-1]/BUFFER_SIZE-1)*BUFFER_SIZE;

				process.locate = LOCATE_SCROLL;

				hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
			}
			break;

		case WM_KEYDOWN:
			if (IsWindowEnabled(hTrack))
			{
				switch (wParam)
				{
					case VK_LEFT:
						if (WaitForSingleObject(hThread, 0)==WAIT_OBJECT_0)
						{
							Display_Flag = true;

							process.locate = LOCATE_BACKWARD;
							hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
						}
						break;

					case VK_RIGHT:
						if (WaitForSingleObject(hThread, 0)==WAIT_OBJECT_0)
						{
//							Display_Flag = true;
							// Here, I'm using lParam to control whether to display or not.
							// This allows us to seek faster. Set 0 to display (normal mode).
							// Display_Flag = -1 is the fastest, the image is not decoded.
							Display_Flag = lParam == 0 ? 1 : -1;

							process.locate = LOCATE_FORWARD;
							hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
						}
						break;
				}
			}
			break;

		case WM_SIZE:
		// JEANL: I don't see why we should respond to WM_SIZE messages...
#if 0
			if (IsIconic(hWnd))
			{
				if (DDOverlay_Flag && Check_Flag)
					IDirectDrawSurface_UpdateOverlay(lpOverlay, NULL, lpPrimary, NULL, DDOVER_HIDE, NULL);
			}
			else
			{
				ShowInfo(false);
				RefreshWindow(true);
			}
#endif
			break;

		case WM_MOVE:
			if (WaitForSingleObject(hThread, 0)==WAIT_OBJECT_0)
			{
				if (IsIconic(hWnd))
				{
					if (DDOverlay_Flag && Check_Flag)
						IDirectDrawSurface_UpdateOverlay(lpOverlay, NULL, lpPrimary, NULL, DDOVER_HIDE, NULL);
				}
				else
				{
					ShowInfo(false);
					RefreshWindow(false);
				}
			}
			break;

		case WM_PAINT:
			hdc = BeginPaint(hWnd, &ps);
			EndPaint(hWnd, &ps);
			ReleaseDC(hWnd, hdc);
			RefreshWindow(false);
			break;


		case WM_DESTROY:
//jsoto_start
//			strcat(szPath, "DVD2AVIMFC.ini");
			strcat(szPath, "Preview.ini");
//jsoto_end
			GetWindowRect(hWnd, &wrect);

			// Let's stop first!
			Stop_Flag = true;
// JEANL: What's up with that?! Removing the sleep, and replacing with a wait for thread.
//			Sleep(500);
			PreviewWaitForThread();

			if (0 && (INIFile = fopen(szPath, "w")) != NULL)
			{
				fprintf(INIFile, "DVD2AVIMFC_Version=%s\n", Version);
				fprintf(INIFile, "Window_Position=%d,%d\n", wrect.left, wrect.top);
				fprintf(INIFile, "iDCT_Algorithm=%d\n", iDCT_Flag);
				fprintf(INIFile, "YUVRGB_Scale=%d\n", Scale_Flag);
				fprintf(INIFile, "Field_Operation=%d\n", FO_Flag);
				fprintf(INIFile, "Output_Method=%d\n", Method_Flag);
				fprintf(INIFile, "Track_Number=%d\n", Track_Flag);
				fprintf(INIFile, "DR_Control=%d\n", DRC_Flag);
				fprintf(INIFile, "DS_Downmix=%d\n", DSDown_Flag);
				fprintf(INIFile, "SRC_Precision=%d\n", SRC_Flag);
				fprintf(INIFile, "Norm_Ratio=%d\n", 100 * Norm_Flag + Norm_Ratio);
				fprintf(INIFile, "Process_Priority=%d\n", Priority_Flag);
				fprintf(INIFile, "Local_Language=%d\n", Language_Flag);

				fclose(INIFile);
			}

			while (File_Limit)
			{
				File_Limit--;
				_close(Infile[File_Limit]);
			}

			Recovery();

			for (i=0; i<8; i++)
				_aligned_free(block[i]);

			for (i=0; i<MAX_FILE_NUMBER; i++)
				free(Infilename[i]);

			if (hLibrary)
				FreeLibrary(hLibrary);

			ReleaseDC(hWnd, hDC);
// jsoto: Deleted PostQuitMessage 
//			PostQuitMessage(0);
/*			DestroyWindow(hTrack);
			DestroyWindow(hLeftButton);
			DestroyWindow(hLeftArrow);
			DestroyWindow(hRightArrow);
			DestroyWindow(hRightButton);
*/
			DefWindowProc(hWnd, message, wParam, lParam);
			break;

		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return false;
}

LRESULT CALLBACK VideoList(HWND hVideoListDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	int i, j;

	switch (message)
	{
		case WM_INITDIALOG:
			if (File_Limit)
				for (i=0; i<File_Limit; i++)
					SendDlgItemMessage(hVideoListDlg, IDC_LIST, LB_ADDSTRING, 0, (LPARAM)Infilename[i]);
			else
				OpenVideoFile(hVideoListDlg);

			if (File_Limit)
				SendDlgItemMessage(hVideoListDlg, IDC_LIST, LB_SETCURSEL, File_Limit-1, 0);
			return true;

		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case ID_ADD:
					AutoIncrement = 0;
					OpenVideoFile(hVideoListDlg);

					if (File_Limit)
						SendDlgItemMessage(hVideoListDlg, IDC_LIST, LB_SETCURSEL, File_Limit-1, 0);
					break;

				case ID_ADD_AUTO:
					AutoIncrement = 1;
					OpenVideoFile(hVideoListDlg);

					if (File_Limit)
						SendDlgItemMessage(hVideoListDlg, IDC_LIST, LB_SETCURSEL, File_Limit-1, 0);
					break;

				case ID_DEL:
					if (File_Limit)
					{
						i= SendDlgItemMessage(hVideoListDlg, IDC_LIST, LB_GETCURSEL, 0, 0);
						SendDlgItemMessage(hVideoListDlg, IDC_LIST, LB_DELETESTRING, i, 0);

						File_Limit--;
						_close(Infile[i]);

						for (j=i; j<File_Limit; j++)
						{
							Infile[j] = Infile[j+1];
							strcpy(Infilename[j], Infilename[j+1]);
						}

						SendDlgItemMessage(hVideoListDlg, IDC_LIST, LB_SETCURSEL, i>=File_Limit ? File_Limit-1 : i, 0);
					}
					break;

				case ID_DELALL:
					while (File_Limit)
					{
						File_Limit--;
						i= SendDlgItemMessage(hVideoListDlg, IDC_LIST, LB_GETCURSEL, 0, 0);
						SendDlgItemMessage(hVideoListDlg, IDC_LIST, LB_DELETESTRING, i, 0);

						_close(Infile[i]);

						for (j=i; j<File_Limit; j++)
						{
							Infile[j] = Infile[j+1];
							strcpy(Infilename[j], Infilename[j+1]);
						}

						SendDlgItemMessage(hVideoListDlg, IDC_LIST, LB_SETCURSEL, i>=File_Limit ? File_Limit-1 : i, 0);
					}
					break;

				case IDOK:
				case IDCANCEL:
					EndDialog(hVideoListDlg, 0);
					Recovery();

					if (File_Limit)
					{
						FileLoadedEnables();
						process.rightfile = File_Limit-1;
						process.rightlba = (int)(process.length[File_Limit-1]/BUFFER_SIZE);

						process.end = process.total - BUFFER_SIZE;
						process.endfile = File_Limit - 1;
						process.endloc = (process.length[File_Limit-1]/BUFFER_SIZE - 1)*BUFFER_SIZE;

						process.locate = LOCATE_INIT;

						if (!threadId || WaitForSingleObject(hThread, INFINITE)==WAIT_OBJECT_0)
							hThread = CreateThread(NULL, 0, MPEG2Dec, 0, 0, &threadId);
					}
					else
					{
						StartupEnables();
					}
					return true;
			}
			break;
	}
    return false;
}

static void OpenVideoFile(HWND hVideoListDlg)
{
	if (PopFileDlg(szInput, hVideoListDlg, OPEN_VOB))
	{
		char *p, *q;
		int val;
		char suffix[_MAX_PATH];
		struct _finddata_t seqfile;

		while (_findfirst(szInput, &seqfile) != -1L)
		{
			SendDlgItemMessage(hVideoListDlg, IDC_LIST, LB_ADDSTRING, 0, (LPARAM)szInput);
			strcpy(Infilename[File_Limit], szInput);
			Infile[File_Limit] = _open(szInput, _O_RDONLY | _O_BINARY | _O_SEQUENTIAL);
			File_Limit++;
			if (AutoIncrement == 0) break;

			// First scan back from the end of the name for an _ character.
			p = szInput+strlen(szInput);
			while (*p != '_' && p >= szInput) p--;
			if (*p != '_') break;
			// Now pick up the number value and increment it.
			p++;
			if (*p < '0' || *p > '9') break;
			sscanf(p, "%d", &val);
			val++;
			// Save the suffix after the number.
			q = p;
			while (*p >= '0' && *p <= '9') p++;
			strcpy(suffix, p);
			// Write the new incremented number.
			sprintf(q, "%d", val);
			// Append the saved suffix.
			strcat(szInput, suffix);
		}
	}
}

LRESULT CALLBACK AudioList(HWND hAudioListDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	int i, j;

	switch (message)
	{
		case WM_INITDIALOG:
			File_Limit = 0;
			OpenAudioFile(hAudioListDlg);
			return true;

		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case ID_ADD:
					OpenAudioFile(hAudioListDlg);
					break;

				case ID_DEL:
					if (File_Limit)
					{
						i= SendDlgItemMessage(hAudioListDlg, IDC_LIST, LB_GETCURSEL, 0, 0);
						SendDlgItemMessage(hAudioListDlg, IDC_LIST, LB_DELETESTRING, i, 0);

						File_Limit--;

						for (j=i; j<File_Limit; j++)
						{
							strcpy(Infilename[j], Infilename[j+1]);
							strcpy(Outfilename[j], Outfilename[j+1]);
							SoundDelay[j] = SoundDelay[j+1];
						}

						SendDlgItemMessage(hAudioListDlg, IDC_LIST, LB_SETCURSEL, i>=File_Limit ? File_Limit-1 : i, 0);
					}
					break;

				case IDOK:
				case IDCANCEL:
					EndDialog(hAudioListDlg, 0);

					if (File_Limit)
					{
// jsoto: Comment next line
//						ShowInfo(true);

						if (!threadId || WaitForSingleObject(hThread, INFINITE)==WAIT_OBJECT_0)
							hThread = CreateThread(NULL, 0, ProcessWAV, (void *)File_Limit, 0, &threadId);
					}
					return true;
			}
			break;
	}
    return false;
}

static void OpenAudioFile(HWND hAudioListDlg)
{
	if (PopFileDlg(szInput, hAudioListDlg, OPEN_WAV))
	{
		if (!CheckWAV())
		{
			sprintf(szBuffer, "Support 16-bit 48KHz or 44.1KHz Stereo Linear PCM Only! ");
			MessageBox(hWnd, szBuffer, NULL, MB_OK | MB_ICONERROR);
			return;
		}

		if (PopFileDlg(szOutput, hAudioListDlg, SAVE_WAV))
		{
			strcpy(Infilename[File_Limit], szInput);
			strcpy(Outfilename[File_Limit], szOutput);
			DialogBox(hInst, (LPCTSTR)IDD_DELAY, hWnd, (DLGPROC)Delay);
			sprintf(szBuffer, "%s %dms", szInput, SoundDelay[File_Limit]);
			SendDlgItemMessage(hAudioListDlg, IDC_LIST, LB_ADDSTRING, 0, (LPARAM)szBuffer);
			File_Limit++;
		}
	}

	if (File_Limit)
		SendDlgItemMessage(hAudioListDlg, IDC_LIST, LB_SETCURSEL, File_Limit-1, 0);
}

DWORD WINAPI ProcessWAV(LPVOID n)
{
	int i;

	Stop_Flag = Pause_Flag = false;
	RunningEnables();

	if (!n)
		Wavefs44File(SoundDelay[0]);
	else
		for (i=0; i<File_Limit && !Stop_Flag; i++)
		{
			strcpy(szInput, Infilename[i]);
			strcpy(szOutput, Outfilename[i]);
			Wavefs44File(SoundDelay[i]);
		}

	File_Limit = 0;

	if (!Stop_Flag)
	{
		MessageBeep(MB_OK);
		SetDlgItemText(hDlg, IDC_REMAIN, "FINISH");
	}

	SetForegroundWindow(hWnd);
	StartupEnables();
	return 0;
}

void ThreadKill()
{
	if ((AVI_Flag || D2V_Flag) && (ac3[Track_Flag].rip && Method_Flag==AUDIO_DECODE  ||  pcm.rip))
	{
		if (SRC_Flag)
		{
			EndSRC(pcm.file);
			pcm.size = ((int)(0.91875*pcm.size)>>2)<<2;
		}

		Normalize(NULL, 44, pcm.filename, pcm.file, 44, pcm.size);
		CloseWAV(pcm.file, pcm.size);
	}

	if (process.locate==LOCATE_INIT || process.locate==LOCATE_RIP)
	{
		if (D2V_Flag)
		{
			// Revised by Donald Graft to support IBBPIBBP...
			WriteGopLine(1);
			if (Stop_Flag)
				fprintf(D2VFile, "\nINTERRUPTED");
			else
				fprintf(D2VFile, "\nFINISHED");

			if (D2V_176_Style == true) 
				fprintf(D2VFile, "\n");
			else
				fprintf(D2VFile, "  %.2f%% FILM\n", (FILM_Purity*100.0)/(FILM_Purity+NTSC_Purity));
		}

		_fcloseall();

		if (Decision_Flag)
		{
			if (Sound_Max > 1)
			{
				PreScale_Ratio = 327.68 * Norm_Ratio / Sound_Max;

				if (PreScale_Ratio > 1.0 && PreScale_Ratio < 1.01)
					PreScale_Ratio = 1.0;

				sprintf(szBuffer, "%.2f", PreScale_Ratio);
				SetDlgItemText(hDlg, IDC_INFO, szBuffer);

				CheckMenuItem(hMenu, IDM_PRESCALE, MF_CHECKED);
				CheckMenuItem(hMenu, IDM_NORM, MF_UNCHECKED);
				Norm_Flag = false;
			}
			else
			{
				SetDlgItemText(hDlg, IDC_INFO, "N.A.");
				CheckMenuItem(hMenu, IDM_PRESCALE, MF_UNCHECKED);
			}
		}

		AVI_Flag = false;
		D2V_Flag = false;
		Decision_Flag = false;
		Display_Flag = false;

		FileLoadedEnables();
		SendMessage(hTrack, TBM_SETSEL, (WPARAM) true, (LPARAM) MAKELONG(process.trackleft, process.trackright));
	}

	if (process.locate==LOCATE_RIP)
	{
		SetForegroundWindow(hWnd);

		if (!Stop_Flag)
		{
//			MessageBeep(MB_OK);	
			SetDlgItemText(hDlg, IDC_REMAIN, "FINISH");
			if (ExitOnEnd) exit(0);
		}
	}

	// We post a message when our thread exits. Normally it's just a ID_THREAD_STOPPED message,
	// but if we hit a sector stop defined by Stop_LBA, we'll send a ID_SECTOR_HIT
	PostMessage(ParenthWnd, WM_USER, (SectorHit) ? ID_SECTOR_HIT : ID_THREAD_STOPPED, 0);
	SectorHit = 0;

	if (CLIActive) SendMessage(hWnd, WM_USER, 0xdeadbeef, 0);
	ExitThread(0);
}

LRESULT CALLBACK Info(HWND hInfoDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG:
			SendDlgItemMessage(hInfoDlg, IDC_PROGRESS, PBM_SETRANGE, 0, MAKELPARAM(0, 10000));
			return true;

		case WM_COMMAND:
			if (LOWORD(wParam)==IDCANCEL)
			{
				DestroyWindow(hInfoDlg);
				Info_Flag = false;
				return true;
			}
		case WM_EXITSIZEMOVE:
			if(!FloatingInfo)
			{
				// If info window was moved and its position isn't "floating", let's save the
				// new position and send a message to the parent window.
				RECT WinRect;
				::GetWindowRect(hInfoDlg,&WinRect);
				InfoXPos = WinRect.left;
				InfoYPos = WinRect.top;
				::SendMessage(ParenthWnd,WM_EXITSIZEMOVE,0,0);
			}
			return true;
	}
    return false;
}

LRESULT CALLBACK Delay(HWND hDelayDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG:
			SetDlgItemText(hDelayDlg, IDC_DELAY, "0");
			return true;

		case WM_COMMAND:
			if (LOWORD(wParam)==IDOK || LOWORD(wParam)==IDCANCEL) 
			{
				SoundDelay[File_Limit] = GetDlgItemInt(hDelayDlg, IDC_DELAY, NULL, true);
				if (abs(SoundDelay[File_Limit]) > 10000000)
					SoundDelay[File_Limit] = 0;

				EndDialog(hDelayDlg, 0);
				return true;
			}
	}
    return false;
}

LRESULT CALLBACK About(HWND hAboutDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG:
			sprintf(szBuffer, "%s", Version);
			SetDlgItemText(hAboutDlg, IDC_VERSION, szBuffer);
			return true;

		case WM_COMMAND:
			if (LOWORD(wParam)==IDOK || LOWORD(wParam)==IDCANCEL) 
			{
				EndDialog(hAboutDlg, 0);
				return true;
			}
	}
    return false;
}

LRESULT CALLBACK ClipResize(HWND hDialog, UINT message, WPARAM wParam, LPARAM lParam)
{
	int i;

	switch (message)
	{
		case WM_INITDIALOG:
			SendDlgItemMessage(hDialog, IDC_LEFT_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, 256));
			SendDlgItemMessage(hDialog, IDC_LEFT_SLIDER, TBM_SETPOS, 1, Clip_Left/2);
			sprintf(szTemp, "%d", Clip_Left);
			SetDlgItemText(hDialog, IDC_LEFT, szTemp);

			SendDlgItemMessage(hDialog, IDC_RIGHT_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, 256));
			SendDlgItemMessage(hDialog, IDC_RIGHT_SLIDER, TBM_SETPOS, 1, Clip_Right/2);
			sprintf(szTemp, "%d", Clip_Right);
			SetDlgItemText(hDialog, IDC_RIGHT, szTemp);

			SendDlgItemMessage(hDialog, IDC_TOP_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, 128));
			SendDlgItemMessage(hDialog, IDC_TOP_SLIDER, TBM_SETPOS, 1, Clip_Top/2);
			sprintf(szTemp, "%d", Clip_Top);
			SetDlgItemText(hDialog, IDC_TOP, szTemp);

			SendDlgItemMessage(hDialog, IDC_BOTTOM_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, 128));
			SendDlgItemMessage(hDialog, IDC_BOTTOM_SLIDER, TBM_SETPOS, 1, Clip_Bottom/2);
			sprintf(szTemp, "%d", Clip_Bottom);
			SetDlgItemText(hDialog, IDC_BOTTOM, szTemp);

			SendDlgItemMessage(hDialog, IDC_WIDTH_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, (Coded_Picture_Width-Clip_Left-Clip_Right)/8));
			SendDlgItemMessage(hDialog, IDC_WIDTH_SLIDER, TBM_SETPOS, 1, (Coded_Picture_Width-Clip_Left-Clip_Right-Squeeze_Width)/8);
			sprintf(szTemp, "%d", Coded_Picture_Width-Clip_Left-Clip_Right-Squeeze_Width);
			SetDlgItemText(hDialog, IDC_WIDTH, szTemp);

			SendDlgItemMessage(hDialog, IDC_HEIGHT_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, (Coded_Picture_Height-Clip_Top-Clip_Bottom)/8));
			SendDlgItemMessage(hDialog, IDC_HEIGHT_SLIDER, TBM_SETPOS, 1, (Coded_Picture_Height-Clip_Top-Clip_Bottom-Squeeze_Height)/8);
			sprintf(szTemp, "%d", Coded_Picture_Height-Clip_Top-Clip_Bottom-Squeeze_Height);
			SetDlgItemText(hDialog, IDC_HEIGHT, szTemp);

			ShowWindow(hDialog, SW_SHOW);

			if (ClipResize_Flag)
				SendDlgItemMessage(hDialog, IDC_CLIPRESIZE_CHECK, BM_SETCHECK, BST_CHECKED, 0);
			return true;

		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case IDC_CLIPRESIZE_CHECK:
					if (SendDlgItemMessage(hDialog, IDC_CLIPRESIZE_CHECK, BM_GETCHECK, 1, 0)==BST_CHECKED)
					{
						CheckMenuItem(hMenu, IDM_CLIPRESIZE, MF_CHECKED);
						ClipResize_Flag = true;
					}
					else
					{
						CheckMenuItem(hMenu, IDM_CLIPRESIZE, MF_UNCHECKED);
						ClipResize_Flag = false;
					}

					RefreshWindow(true);
					ShowInfo(false);
					break;

				case IDCANCEL:
					EndDialog(hDialog, 0);
					return true;
			}
			break;

		case WM_HSCROLL:
			switch (GetWindowLong((HWND)lParam, GWL_ID))
			{
				case IDC_LEFT_SLIDER:
					i = SendDlgItemMessage(hDialog, IDC_LEFT_SLIDER, TBM_GETPOS, 0, 0)*2;
					if (i+Clip_Right+MIN_WIDTH <= Coded_Picture_Width)
					{
						Clip_Left = i;
						sprintf(szTemp, "%d", Clip_Left);
						SetDlgItemText(hDialog, IDC_LEFT, szTemp);

						Clip_Right = 8 - Clip_Left%8;
						sprintf(szTemp, "%d", Clip_Right);
						SetDlgItemText(hDialog, IDC_RIGHT, szTemp);
						SendDlgItemMessage(hDialog, IDC_RIGHT_SLIDER, TBM_SETPOS, 1, Clip_Right/2);

						Squeeze_Width = 0;
						SendDlgItemMessage(hDialog, IDC_WIDTH_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, (Coded_Picture_Width-Clip_Left-Clip_Right)/8));
						SendDlgItemMessage(hDialog, IDC_WIDTH_SLIDER, TBM_SETPOS, 1, (Coded_Picture_Width-Clip_Left-Clip_Right-Squeeze_Width)/8);
						sprintf(szTemp, "%d", Coded_Picture_Width-Clip_Left-Clip_Right-Squeeze_Width);
						SetDlgItemText(hDialog, IDC_WIDTH, szTemp);
					}
					break;

				case IDC_RIGHT_SLIDER:
					i = SendDlgItemMessage(hDialog, IDC_RIGHT_SLIDER, TBM_GETPOS, 0, 0)*2;
					if (i+Clip_Left+MIN_WIDTH<=Coded_Picture_Width && (i+Clip_Left)%8==0)
					{
						Clip_Right = i;
						sprintf(szTemp, "%d", Clip_Right);
						SetDlgItemText(hDialog, IDC_RIGHT, szTemp);

						Squeeze_Width = 0;
						SendDlgItemMessage(hDialog, IDC_WIDTH_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, (Coded_Picture_Width-Clip_Left-Clip_Right)/8));
						SendDlgItemMessage(hDialog, IDC_WIDTH_SLIDER, TBM_SETPOS, 1, (Coded_Picture_Width-Clip_Left-Clip_Right-Squeeze_Width)/8);
						sprintf(szTemp, "%d", Coded_Picture_Width-Clip_Left-Clip_Right-Squeeze_Width);
						SetDlgItemText(hDialog, IDC_WIDTH, szTemp);
					}
					break;

				case IDC_TOP_SLIDER:
					i = SendDlgItemMessage(hDialog, IDC_TOP_SLIDER, TBM_GETPOS, 0, 0)*2;
					if (i+Clip_Bottom+MIN_HEIGHT <= Coded_Picture_Height)
					{
						Clip_Top = i;
						sprintf(szTemp, "%d", Clip_Top);
						SetDlgItemText(hDialog, IDC_TOP, szTemp);

						Clip_Bottom = 8 - Clip_Top%8;
						sprintf(szTemp, "%d", Clip_Bottom);
						SetDlgItemText(hDialog, IDC_BOTTOM, szTemp);
						SendDlgItemMessage(hDialog, IDC_BOTTOM_SLIDER, TBM_SETPOS, 1, Clip_Bottom/2);

						Squeeze_Height = 0;
						SendDlgItemMessage(hDialog, IDC_HEIGHT_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, (Coded_Picture_Height-Clip_Top-Clip_Bottom)/8));
						SendDlgItemMessage(hDialog, IDC_HEIGHT_SLIDER, TBM_SETPOS, 1, (Coded_Picture_Height-Clip_Top-Clip_Bottom-Squeeze_Height)/8);
						sprintf(szTemp, "%d", Coded_Picture_Height-Clip_Top-Clip_Bottom-Squeeze_Height);
						SetDlgItemText(hDialog, IDC_HEIGHT, szTemp);
					}
					break;

				case IDC_BOTTOM_SLIDER:
					i = SendDlgItemMessage(hDialog, IDC_BOTTOM_SLIDER, TBM_GETPOS, 0, 0)*2;
					if (i+Clip_Top+MIN_HEIGHT<=Coded_Picture_Height && (i+Clip_Top)%8==0)
					{
						Clip_Bottom = i;
						sprintf(szTemp, "%d", Clip_Bottom);
						SetDlgItemText(hDialog, IDC_BOTTOM, szTemp);

						Squeeze_Height = 0;
						SendDlgItemMessage(hDialog, IDC_HEIGHT_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, (Coded_Picture_Height-Clip_Top-Clip_Bottom)/8));
						SendDlgItemMessage(hDialog, IDC_HEIGHT_SLIDER, TBM_SETPOS, 1, (Coded_Picture_Height-Clip_Top-Clip_Bottom-Squeeze_Height)/8);
						sprintf(szTemp, "%d", Coded_Picture_Height-Clip_Top-Clip_Bottom-Squeeze_Height);
						SetDlgItemText(hDialog, IDC_HEIGHT, szTemp);
					}
					break;

				case IDC_WIDTH_SLIDER:
					i = Coded_Picture_Width - Clip_Left - Clip_Right - (SendDlgItemMessage(hDialog, IDC_WIDTH_SLIDER, TBM_GETPOS, 0, 0)*8);
					if (MIN_WIDTH+i+Clip_Left+Clip_Right <= Coded_Picture_Width)
					{
						Squeeze_Width = i;
						sprintf(szTemp, "%d", Coded_Picture_Width - Clip_Left - Clip_Right - Squeeze_Width);
						SetDlgItemText(hDialog, IDC_WIDTH, szTemp);
					}
					break;

				case IDC_HEIGHT_SLIDER:
					i = Coded_Picture_Height - Clip_Top - Clip_Bottom - (SendDlgItemMessage(hDialog, IDC_HEIGHT_SLIDER, TBM_GETPOS, 0, 0)*8);
					if (MIN_HEIGHT+i+Clip_Top+Clip_Bottom <= Coded_Picture_Height)
					{
						Squeeze_Height = i;
						sprintf(szTemp, "%d", Coded_Picture_Height - Clip_Top - Clip_Bottom - Squeeze_Height);
						SetDlgItemText(hDialog, IDC_HEIGHT, szTemp);
					}
					break;
			}

			RefreshWindow(true);
			ShowInfo(false);
			break;
	}
    return false;
}

LRESULT CALLBACK Luminance(HWND hDialog, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG:
			SendDlgItemMessage(hDialog, IDC_GAMMA_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, 512));
			SendDlgItemMessage(hDialog, IDC_GAMMA_SLIDER, TBM_SETTICFREQ, 256, 0);
			SendDlgItemMessage(hDialog, IDC_GAMMA_SLIDER, TBM_SETPOS, 1, LumGamma+256);
			sprintf(szTemp, "%d", LumGamma);
			SetDlgItemText(hDialog, IDC_GAIN, szTemp);

			SendDlgItemMessage(hDialog, IDC_OFFSET_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, 512));
			SendDlgItemMessage(hDialog, IDC_OFFSET_SLIDER, TBM_SETTICFREQ, 256, 0);
			SendDlgItemMessage(hDialog, IDC_OFFSET_SLIDER, TBM_SETPOS, 1, LumOffset+256);
			sprintf(szTemp, "%d", LumOffset);
			SetDlgItemText(hDialog, IDC_OFFSET, szTemp);

			ShowWindow(hDialog, SW_SHOW);

			if (Luminance_Flag)
				SendDlgItemMessage(hDialog, IDC_LUM_CHECK, BM_SETCHECK, BST_CHECKED, 0);
			return true;

		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case IDC_LUM_CHECK:
					if (SendDlgItemMessage(hDialog, IDC_LUM_CHECK, BM_GETCHECK, 1, 0)==BST_CHECKED)
					{
						CheckMenuItem(hMenu, IDM_LUMINANCE, MF_CHECKED);
						Luminance_Flag = true;
					}
					else
					{
						CheckMenuItem(hMenu, IDM_LUMINANCE, MF_UNCHECKED);
						Luminance_Flag = false;
					}
					
					RefreshWindow(true);
					break;

				case IDCANCEL:
					EndDialog(hDialog, 0);
					return true;
			}
			break;

		case WM_HSCROLL:
			switch (GetWindowLong((HWND)lParam, GWL_ID))
			{
				case IDC_GAMMA_SLIDER:
					LumGamma = SendDlgItemMessage(hDialog, IDC_GAMMA_SLIDER, TBM_GETPOS, 0, 0) - 256;
					sprintf(szTemp, "%d", LumGamma);
					SetDlgItemText(hDialog, IDC_GAIN, szTemp);	
					break;

				case IDC_OFFSET_SLIDER:
					LumOffset = SendDlgItemMessage(hDialog, IDC_OFFSET_SLIDER, TBM_GETPOS, 0, 0) - 256;
					sprintf(szTemp, "%d", LumOffset);
					SetDlgItemText(hDialog, IDC_OFFSET, szTemp);
					break;
			}

			RefreshWindow(true);
			break;
	}
    return false;
}

LRESULT CALLBACK Normalization(HWND hDialog, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG:
			SendDlgItemMessage(hDialog, IDC_NORM_SLIDER, TBM_SETRANGE, 0, MAKELPARAM(0, 100));
			SendDlgItemMessage(hDialog, IDC_NORM_SLIDER, TBM_SETTICFREQ, 50, 0);
			SendDlgItemMessage(hDialog, IDC_NORM_SLIDER, TBM_SETPOS, 1, Norm_Ratio);
			sprintf(szTemp, "%d", Norm_Ratio);
			SetDlgItemText(hDialog, IDC_NORM, szTemp);

			ShowWindow(hDialog, SW_SHOW);

			if (Norm_Flag)
				SendDlgItemMessage(hDialog, IDC_NORM_CHECK, BM_SETCHECK, BST_CHECKED, 0);
			return true;

		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case IDC_NORM_CHECK:
					if (SendDlgItemMessage(hDialog, IDC_NORM_CHECK, BM_GETCHECK, 1, 0)==BST_CHECKED)
					{
						CheckMenuItem(hMenu, IDM_NORM, MF_CHECKED);
						Norm_Flag = true;
					}
					else
					{
						CheckMenuItem(hMenu, IDM_NORM, MF_UNCHECKED);
						Norm_Flag = false;
					}
					break;

				case IDCANCEL:
					EndDialog(hDialog, 0);
					return true;
			}
			break;

		case WM_HSCROLL:
			if (GetWindowLong((HWND)lParam, GWL_ID)==IDC_NORM_SLIDER)
			{
				Norm_Ratio = SendDlgItemMessage(hDialog, IDC_NORM_SLIDER, TBM_GETPOS, 0, 0);
				sprintf(szTemp, "%d", Norm_Ratio);
				SetDlgItemText(hDialog, IDC_NORM, szTemp);
			}
			break;
	}
    return false;
}

/* register the window class */
ATOM MyRegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize			= sizeof(WNDCLASSEX);
	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)DVD2AVIWndProc;
	wcex.cbClsExtra		= false;
	wcex.cbWndExtra		= false;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, (LPCTSTR)IDI_MOVIE);
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= CreateSolidBrush(MASKCOLOR);

//jsoto_start
	wcex.lpszMenuName = NULL;
//jsoto_end
/*
	switch(GetUserDefaultLangID())
	{
		case 0x0404:
			wcex.lpszMenuName = (LPCSTR)IDC_GUI_TW;
			break;

		case 0x0411:
			wcex.lpszMenuName = (LPCSTR)IDC_GUI_JP;
			break;

		default:
			wcex.lpszMenuName = (LPCSTR)IDC_GUI;
			break;
	}

	switch(Language_Flag)
	{
		case LANGUAGE_CHINESE:
			wcex.lpszMenuName = (LPCSTR)IDC_GUI_TW;
			break;

		case LANGUAGE_ENGLISH:
			wcex.lpszMenuName = (LPCSTR)IDC_GUI;
			break;

		case LANGUAGE_JAPANESE:
			wcex.lpszMenuName = (LPCSTR)IDC_GUI_JP;
			break;
	}
*/
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, (LPCTSTR)IDI_SMALL);

	return RegisterClassEx(&wcex);
}

static char *ExtFilter[4] = 
{
	".avi", ".d2v", ".wav", ".bmp"
};

bool PopFileDlg(PTSTR pstrFileName, HWND hOwner, int Status)
{
	static OPENFILENAME ofn;
	static char *szFilter, *ext;

	int count = 0;

	switch (Status)
	{
		case OPEN_VOB:
			szFilter = TEXT ("MPEG-2 Stream (*.vob; *.m2p; *.m2v; *.mpv)\0*.vob;*.m2p;*.m2v;*.mpv\0")  \
				TEXT ("All Files (*.*)\0*.*\0");
			break;

		case SAVE_AVI:
			szFilter = TEXT ("AVI File (*.avi)\0*.avi;*.ac3;*.wav;*.mpa;*.dts\0")  \
				TEXT ("All Files (*.*)\0*.*\0");
			break;

		case OPEN_D2V:
			szFilter = TEXT ("DVD2AVIMFC Project File (*.d2v)\0*.d2v\0")  \
				TEXT ("All Files (*.*)\0*.*\0");
			break;

		case SAVE_D2V:
			szFilter = TEXT ("DVD2AVIMFC Project File (*.d2v)\0*.d2v;*.ac3;*.wav;*.mpa;*.dts\0")  \
				TEXT ("All Files (*.*)\0*.*\0");
			break;

		case SAVE_BMP:
			szFilter = TEXT ("BMP File (*.bmp)\0*.bmp\0")  \
				TEXT ("All Files (*.*)\0*.*\0");
			break;

		case OPEN_WAV:
		case SAVE_WAV:
			szFilter = TEXT ("WAV File (*.wav)\0*.wav\0")  \
				TEXT ("All Files (*.*)\0*.*\0");
			break;
	}

	ofn.lStructSize       = sizeof (OPENFILENAME) ;
	ofn.hwndOwner         = hOwner ;
	ofn.hInstance         = hInst ;
	ofn.lpstrFilter       = szFilter ;
	ofn.nMaxFile          = _MAX_PATH ;
	ofn.nMaxFileTitle     = _MAX_PATH ;
	ofn.lpstrFile         = pstrFileName ;

	switch (Status)
	{
		case OPEN_VOB:
		case OPEN_D2V:
		case OPEN_WAV:
			gop_warned = false;
			ofn.Flags = OFN_HIDEREADONLY | OFN_FILEMUSTEXIST;
			return GetOpenFileName(&ofn);

		case SAVE_BMP:
			count++;
		case SAVE_WAV:
			count++;
		case SAVE_D2V:
			count++;
		case SAVE_AVI:
			ofn.Flags = OFN_HIDEREADONLY | OFN_ALLOWMULTISELECT | OFN_EXPLORER;
			if (GetSaveFileName(&ofn))
			{
				ext = strrchr(pstrFileName, '.');
				if (ext!=NULL && !_strnicmp(ext, ExtFilter[count], 4))
					strncpy(ext, ext+4, 1);
				return true;
			}
	}
	return false;
}


static void ShowInfo(bool update)
{
	if (update)
	{
		if (Info_Flag)
			DestroyWindow(hDlg);

		Info_Flag = true;
		hDlg = CreateDialog(hInst, (LPCTSTR)IDD_INFO, hWnd, (DLGPROC)Info);
	}

	if (Info_Flag)
	{
		GetWindowRect(hDlg, &crect);
		GetWindowRect(hWnd, &wrect);
		if(FloatingInfo)
		MoveWindow(hDlg, wrect.right, wrect.top+Edge_Height-Edge_Width/2, crect.right-crect.left, crect.bottom-crect.top, true);
		else
			MoveWindow(hDlg, InfoXPos, InfoYPos, crect.right-crect.left, crect.bottom-crect.top, true);
		ShowWindow(hDlg, SW_SHOW);
	}
}

static void ClearTrack()
{
	CheckMenuItem(hMenu, IDM_TRACK_1, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TRACK_2, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TRACK_3, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TRACK_4, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TRACK_5, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TRACK_6, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TRACK_7, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TRACK_8, MF_UNCHECKED);
}

static void CheckFlag()
{
	CheckMenuItem(hMenu, IDM_KEY_OFF, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_KEY_INPUT, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_KEY_OP, MF_UNCHECKED);

	if (!OpenDVD_Flag)
		KeyOp_Flag = KEY_OFF;

	switch (KeyOp_Flag)
	{
		case KEY_OFF:
			CheckMenuItem(hMenu, IDM_KEY_OFF, MF_CHECKED);
			break;

		case KEY_INPUT:
			CheckMenuItem(hMenu, IDM_KEY_INPUT, MF_CHECKED);
			break;

		case KEY_OP:
			CheckMenuItem(hMenu, IDM_KEY_OP, MF_CHECKED);
			break;
	}

	CheckMenuItem(hMenu, IDM_IDCT_MMX, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_IDCT_SSEMMX, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_IDCT_SSE2MMX, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_IDCT_FPU, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_IDCT_REF, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_PCSCALE, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_TVSCALE, MF_UNCHECKED);
	CheckMenuItem(hMenu, IDM_FO_FILM, MF_UNCHECKED);

	if (iDCT_Flag==IDCT_MMX)
	{
		if (cpu.ssemmx)
			iDCT_Flag = IDCT_SSEMMX;

		if (cpu.sse2)
			iDCT_Flag = IDCT_SSE2MMX;
	}
	else if (iDCT_Flag==IDCT_SSEMMX)
	{
		if (cpu.sse2)
			iDCT_Flag = IDCT_SSE2MMX;

		if (!cpu.ssemmx)
			iDCT_Flag = IDCT_MMX;
	}
	else if (iDCT_Flag==IDCT_SSE2MMX)
	{
		if (!cpu.sse2)
			iDCT_Flag = IDCT_SSEMMX;

		if (!cpu.ssemmx)
			iDCT_Flag = IDCT_MMX;
	}

	switch (iDCT_Flag)
	{
		case IDCT_MMX:
			CheckMenuItem(hMenu, IDM_IDCT_MMX, MF_CHECKED);
			break;

		case IDCT_SSEMMX:
			CheckMenuItem(hMenu, IDM_IDCT_SSEMMX, MF_CHECKED);
			break;

		case IDCT_SSE2MMX:
			CheckMenuItem(hMenu, IDM_IDCT_SSE2MMX, MF_CHECKED);
			break;

		case IDCT_FPU:
			CheckMenuItem(hMenu, IDM_IDCT_FPU, MF_CHECKED);
			break;

		case IDCT_REF:
			CheckMenuItem(hMenu, IDM_IDCT_REF, MF_CHECKED);
			break;
	}

	if (Scale_Flag)
	{
		RGB_Scale = 0x1000254310002543;
		RGB_Offset = 0x0010001000100010;
		RGB_CBU = 0x0000408D0000408D;
		RGB_CGX = 0xF377E5FCF377E5FC;
		RGB_CRV = 0x0000331300003313;

		CheckMenuItem(hMenu, IDM_PCSCALE, MF_CHECKED);
	}
	else
	{
		RGB_Scale = 0x1000200010002000;
		RGB_Offset = 0x0000000000000000;
		RGB_CBU = 0x000038B4000038B4;
		RGB_CGX = 0xF4FDE926F4FDE926;
		RGB_CRV = 0x00002CDD00002CDD;

		CheckMenuItem(hMenu, IDM_TVSCALE, MF_CHECKED);
	}

	switch (FO_Flag)
	{
		case FO_NONE:
			CheckMenuItem(hMenu, IDM_FO_NONE, MF_CHECKED);
			CheckMenuItem(hMenu, IDM_FO_FILM, MF_UNCHECKED);
			CheckMenuItem(hMenu, IDM_FO_SWAP, MF_UNCHECKED);
			break;

		case FO_FILM:
			CheckMenuItem(hMenu, IDM_FO_NONE, MF_UNCHECKED);
			CheckMenuItem(hMenu, IDM_FO_FILM, MF_CHECKED);
			CheckMenuItem(hMenu, IDM_FO_SWAP, MF_UNCHECKED);
			break;

		case FO_SWAP:
			CheckMenuItem(hMenu, IDM_FO_NONE, MF_UNCHECKED);
			CheckMenuItem(hMenu, IDM_FO_FILM, MF_UNCHECKED);
			CheckMenuItem(hMenu, IDM_FO_SWAP, MF_CHECKED);
			break;
	}

	switch (Track_Flag)
	{
		case TRACK_1:
			CheckMenuItem(hMenu, IDM_TRACK_1, MF_CHECKED);
			break;

		case TRACK_2:
			CheckMenuItem(hMenu, IDM_TRACK_2, MF_CHECKED);
			break;

		case TRACK_3:
			CheckMenuItem(hMenu, IDM_TRACK_3, MF_CHECKED);
			break;

		case TRACK_4:
			CheckMenuItem(hMenu, IDM_TRACK_4, MF_CHECKED);
			break;

		case TRACK_5:
			CheckMenuItem(hMenu, IDM_TRACK_5, MF_CHECKED);
			break;

		case TRACK_6:
			CheckMenuItem(hMenu, IDM_TRACK_6, MF_CHECKED);
			break;

		case TRACK_7:
			CheckMenuItem(hMenu, IDM_TRACK_7, MF_CHECKED);
			break;

		case TRACK_8:
			CheckMenuItem(hMenu, IDM_TRACK_8, MF_CHECKED);
			break;
	}

	switch (Method_Flag)
	{
		case AUDIO_NONE:
			CheckMenuItem(hMenu, IDM_AUDIO_NONE, MF_CHECKED);
			break;

		case AUDIO_DEMUX:
			CheckMenuItem(hMenu, IDM_DEMUX, MF_CHECKED);
			break;

		case AUDIO_DEMUXALL:
			CheckMenuItem(hMenu, IDM_DEMUXALL, MF_CHECKED);
			break;

		case AUDIO_DECODE:
			CheckMenuItem(hMenu, IDM_DECODE, MF_CHECKED);
			break;
	}

	switch (DRC_Flag)
	{
		case DRC_NONE:
			CheckMenuItem(hMenu, IDM_DRC_NONE, MF_CHECKED);
			break;

		case DRC_LIGHT:
			CheckMenuItem(hMenu, IDM_DRC_LIGHT, MF_CHECKED);
			break;

		case DRC_NORMAL:
			CheckMenuItem(hMenu, IDM_DRC_NORMAL, MF_CHECKED);
			break;

		case DRC_HEAVY:
			CheckMenuItem(hMenu, IDM_DRC_HEAVY, MF_CHECKED);
			break;
	}

	if (DSDown_Flag)
		CheckMenuItem(hMenu, IDM_DSDOWN, MF_CHECKED);

	switch (SRC_Flag)
	{
		case SRC_NONE:
			CheckMenuItem(hMenu, IDM_SRC_NONE, MF_CHECKED);
			break;

		case SRC_LOW:
			CheckMenuItem(hMenu, IDM_SRC_LOW, MF_CHECKED);
			break;

		case SRC_MID:
			CheckMenuItem(hMenu, IDM_SRC_MID, MF_CHECKED);
			break;

		case SRC_HIGH:
			CheckMenuItem(hMenu, IDM_SRC_HIGH, MF_CHECKED);
			break;

		case SRC_UHIGH:
			CheckMenuItem(hMenu, IDM_SRC_UHIGH, MF_CHECKED);
			break;
	}

	if (Norm_Ratio > 100)
	{
		CheckMenuItem(hMenu, IDM_NORM, MF_CHECKED);
		Norm_Flag = true;
		Norm_Ratio -= 100;
	}

	switch (Priority_Flag)
	{
		case PRIORITY_HIGH:
			CheckMenuItem(hMenu, IDM_PP_HIGH, MF_CHECKED);
			SetPriorityClass(hProcess, HIGH_PRIORITY_CLASS);
			break;

		case PRIORITY_NORMAL:
			CheckMenuItem(hMenu, IDM_PP_NORMAL, MF_CHECKED);
			SetPriorityClass(hProcess, NORMAL_PRIORITY_CLASS);
			break;

		case PRIORITY_LOW:
			CheckMenuItem(hMenu, IDM_PP_LOW, MF_CHECKED);
			SetPriorityClass(hProcess, IDLE_PRIORITY_CLASS);
			break;
	}

	switch (Language_Flag)
	{
		case LANGUAGE_CHINESE:
			CheckMenuItem(hMenu, IDM_LANG_CHINESE, MF_CHECKED);
			break;

		case LANGUAGE_ENGLISH:
			CheckMenuItem(hMenu, IDM_LANG_ENGLISH, MF_CHECKED);
			break;

		case LANGUAGE_JAPANESE:
			CheckMenuItem(hMenu, IDM_LANG_JAPANESE, MF_CHECKED);
			break;

		default:
			Language_Flag = LANGUAGE_DEFAULT;
			CheckMenuItem(hMenu, IDM_LANG_DEFAULT, MF_CHECKED);
			break;
	}
}

// This is a quick shortcut to release the direct-draw ressource as fast as possible.
void ReleaseAll()
{
	// To avoid getting an exception during Recovery().
	DDOverlay_Flag = 0;
	// Now release everything!
	if (lpDD2)
		lpDD2->Release();
	lpDD2 = 0;
	if (lpDD)
		lpDD->Release();
	lpDD = 0;
}	

static void Recovery()
{
	int i;

	if (Check_Flag)
	{
		if (DDOverlay_Flag)
		{
			DDOverlay_Flag = false;
			CheckMenuItem(hMenu, IDM_DIRECTDRAW, MF_UNCHECKED);
			IDirectDrawSurface_UpdateOverlay(lpOverlay, NULL, lpPrimary, NULL, DDOVER_HIDE, NULL);
		}

		if (lpDD2)
			lpDD2->Release();
		if (lpDD)
			lpDD->Release();

		for (i=0; i<3; i++)
		{
			_aligned_free(backward_reference_frame[i]);
			_aligned_free(forward_reference_frame[i]);
			_aligned_free(auxframe[i]);
		}

		_aligned_free(u422);
		_aligned_free(v422);
		_aligned_free(u444);
		_aligned_free(v444);
		_aligned_free(rgb24);
		_aligned_free(rgb24Scaled);
		_aligned_free(yuy2);
		_aligned_free(lum);
	}

	Check_Flag = false;

	SendMessage(hTrack, TBM_SETPOS, (WPARAM) true, 0);
	SendMessage(hTrack, TBM_SETSEL, (WPARAM) true, (LPARAM) MAKELONG(0, 0));

	LumGamma = LumOffset = 0;
	Luminance_Flag = false;
	CheckMenuItem(hMenu, IDM_LUMINANCE, MF_UNCHECKED);

	Clip_Left = Clip_Right = Clip_Top = Clip_Bottom = Squeeze_Width = Squeeze_Height = 0;
	ClipResize_Flag = false;
	CheckMenuItem(hMenu, IDM_CLIPRESIZE, MF_UNCHECKED);

	PreScale_Ratio = 1.0;
	CheckMenuItem(hMenu, IDM_PRESCALE, MF_UNCHECKED);

//jsoto_start
//	SetWindowText(hWnd, "DVD2AVIMFC");
	SetWindowText(hWnd, "VobBlanker Preview");
//jsoto_end
	if (File_Limit)
	{
		ZeroMemory(&process, sizeof(PROCESS));
		process.trackright = TRACK_PITCH;

		SystemStream_Flag = false;
		Display_Flag = true;
		// JEANL: To avoid displaying unless really needed.
		Display_Flag = false;

		CumulStartSectors[0]=0;
		for (i=0; i<File_Limit; i++)
		{
			process.length[i] = _filelengthi64(Infile[i]);
			process.total += process.length[i];
			CumulStartSectors[i+1] = CumulStartSectors[i] + (int)(process.length[i]/BUFFER_SIZE);
		}
	}

//	ResizeWindow(INIT_WIDTH, INIT_HEIGHT);
//	ResizeWindow(INIT_WIDTH, INIT_HEIGHT);	// 2-line menu antidote
}

/*

  -	halcaps	{...}
	dwSize	380
	dwCaps	339901249
	dwCaps2	661504
	dwCKeyCaps	528
	dwFXCaps	44646400
	dwFXAlphaCaps	0
	dwPalCaps	0
	dwSVCaps	0
	dwAlphaBltConstBitDepths	0
	dwAlphaBltPixelBitDepths	0
	dwAlphaBltSurfaceBitDepths	0
	dwAlphaOverlayConstBitDepths	0
	dwAlphaOverlayPixelBitDepths	0
	dwAlphaOverlaySurfaceBitDepths	0
	dwZBufferBitDepths	1024
	dwVidMemTotal	30736384
	dwVidMemFree	28662784
	dwMaxVisibleOverlays	1
	dwCurrVisibleOverlays	0
	dwNumFourCCCodes	6
	dwAlignBoundarySrc	0
	dwAlignSizeSrc	0
	dwAlignBoundaryDest	0
	dwAlignSizeDest	0
	dwAlignStrideAlign	0
+	dwRops	0x00497a3c
+	ddsOldCaps	{...}
	dwMinOverlayStretch	100
	dwMaxOverlayStretch	20000
	dwMinLiveVideoStretch	100
	dwMaxLiveVideoStretch	20000
	dwMinHwCodecStretch	1
	dwMaxHwCodecStretch	4095000
	dwReserved1	0
	dwReserved2	0
	dwReserved3	0
	dwSVBCaps	0
	dwSVBCKeyCaps	0
	dwSVBFXCaps	0
+	dwSVBRops	0x00497a90
	dwVSBCaps	0
	dwVSBCKeyCaps	0
	dwVSBFXCaps	0
+	dwVSBRops	0x00497abc
	dwSSBCaps	0
	dwSSBCKeyCaps	0
	dwSSBFXCaps	0
+	dwSSBRops	0x00497ae8
	dwMaxVideoPorts	0
	dwCurrVideoPorts	0
	dwSVBCaps2	0
	dwNLVBCaps	0
	dwNLVBCaps2	0
	dwNLVBCKeyCaps	0
	dwNLVBFXCaps	0
+	dwNLVBRops	0x00497b24
+	ddsCaps	{...}
+	(halcaps).ddsCaps	{...}

  */

void CheckDirectDraw()
{
	if (DirectDrawCreate(NULL, &lpDD, NULL)==DD_OK)
	{
		if (lpDD->QueryInterface(IID_IDirectDraw2, (LPVOID*)&lpDD2)==DD_OK)
		{
			if (lpDD2->SetCooperativeLevel(hWnd, DDSCL_NORMAL)==DD_OK)
			{
				ZeroMemory(&ddsd, sizeof(DDSURFACEDESC));
				ddsd.dwSize = sizeof(DDSURFACEDESC);
				ddsd.dwFlags = DDSD_CAPS;
				ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_VIDEOMEMORY;

				if (lpDD2->CreateSurface(&ddsd, &lpPrimary, NULL)==DD_OK)
				{
					ZeroMemory(&halcaps, sizeof(DDCAPS));
					halcaps.dwSize = sizeof(DDCAPS);

					if (lpDD2->GetCaps(&halcaps, NULL)==DD_OK)
					{
#ifndef NO_OVERLAY_USE
						if (halcaps.dwCaps & DDCAPS_OVERLAY)
#else
						if(0)
#endif
						{
							DDPIXELFORMAT ddPixelFormat;
							ddPixelFormat.dwFlags = DDPF_FOURCC;
							ddPixelFormat.dwFourCC = mmioFOURCC('Y','U','Y','2');
							ddPixelFormat.dwYUVBitCount = 16;

							ZeroMemory(&ddsd, sizeof(DDSURFACEDESC));
							ddsd.dwSize = sizeof(DDSURFACEDESC);
							ddsd.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT | DDSD_PIXELFORMAT;
							ddsd.ddsCaps.dwCaps = DDSCAPS_OVERLAY | DDSCAPS_VIDEOMEMORY;
							ddsd.dwWidth = Coded_Picture_Width;
							ddsd.dwHeight = Coded_Picture_Height;

							memcpy(&(ddsd.ddpfPixelFormat), &ddPixelFormat, sizeof(DDPIXELFORMAT));
							
							HRESULT hr = lpDD2->CreateSurface(&ddsd, &lpOverlay, NULL);
							bool u = hr == DDERR_OUTOFCAPS;
							// THIS is the error I get: 0x88760168
							if (hr ==DD_OK)
							{
								DDOverlay_Flag = true;

								ZeroMemory(&ddofx, sizeof(DDOVERLAYFX));
								ddofx.dwSize = sizeof(DDOVERLAYFX);

								ddofx.dckDestColorkey.dwColorSpaceLowValue = DDColorMatch(lpPrimary, MASKCOLOR);
								ddofx.dckDestColorkey.dwColorSpaceHighValue = ddofx.dckDestColorkey.dwColorSpaceLowValue;

								if (IDirectDrawSurface_Lock(lpOverlay, NULL, &ddsd, 0, NULL)==DD_OK)
								{
									int i, j;
									unsigned char *dst = (unsigned char *)ddsd.lpSurface;

									for (i=0; i<Coded_Picture_Height; i++)
									{
										for (j=0; j<Coded_Picture_Width; j++)
										{
											dst[j*2] = 0;
											dst[j*2+1] = 128;
										}

										dst += ddsd.lPitch;
									}

									IDirectDrawSurface_Unlock(lpOverlay, NULL);
								}
							}
							// I can't get an overlay surface... 
							// JEANL:
//							else 
//								MessageBox(hWnd,"Can't get overlay!",NULL, MB_OK | MB_ICONERROR);
						}
					}
				}
			}
		}
	}

	if (DDOverlay_Flag)
	{
		Store_Flag = STORE_YUY2;
		CheckMenuItem(hMenu, IDM_STORE_RGB24, MF_UNCHECKED);
		CheckMenuItem(hMenu, IDM_STORE_YUY2, MF_CHECKED);
		CheckMenuItem(hMenu, IDM_DIRECTDRAW, MF_CHECKED);
	}
	else
	{
		Store_Flag = STORE_RGB24;
		CheckMenuItem(hMenu, IDM_STORE_RGB24, MF_CHECKED);
		CheckMenuItem(hMenu, IDM_STORE_YUY2, MF_UNCHECKED);
	}
}

static DWORD DDColorMatch(LPDIRECTDRAWSURFACE pdds, COLORREF rgb)
{
	COLORREF	rgbT;
	DWORD		dw = CLR_INVALID;
	HRESULT		hres;
	HDC			hdc;

	if (IDirectDrawSurface_GetDC(pdds, &hdc)==DD_OK)
	{
		rgbT = GetPixel(hdc, 0, 0);
		SetPixel(hdc, 0, 0, rgb);
		IDirectDrawSurface_ReleaseDC(pdds, hdc);
	}

	ZeroMemory(&ddsd, sizeof(DDSURFACEDESC));
	ddsd.dwSize = sizeof(DDSURFACEDESC);

	while ((hres = IDirectDrawSurface_Lock(pdds, NULL, &ddsd, 0, NULL)) == DDERR_WASSTILLDRAWING)
		;

	if (hres==DD_OK)
	{
		dw = *(DWORD *) ddsd.lpSurface;
		if (ddsd.ddpfPixelFormat.dwRGBBitCount < 32)
			dw &= (1 << ddsd.ddpfPixelFormat.dwRGBBitCount) - 1;
		IDirectDrawSurface_Unlock(pdds, NULL);
	}

	if (IDirectDrawSurface_GetDC(pdds, &hdc)==DD_OK)
	{
		SetPixel(hdc, 0, 0, rgbT);
		IDirectDrawSurface_ReleaseDC(pdds, hdc);
	}

	return dw;
}

void ResizeWindow(int width, int height)
{
	static int Old_aspect_ratio = -1;
	if(AutoScale == 1)
	{

		// If AutoScale is 1, we scale the y-axis according to the aspect ratio, and according
		// to the x-axis scaling.
		switch(aspect_ratio_information)
		{
		case 0:
		case 1: 
		case 2: YUVScaleY = YUVScaleX; break;
		case 3: YUVScaleY = YUVScaleX / 1.33333f; break;
		case 4: YUVScaleY = YUVScaleX / 1.6575f; break;
		}
	}

	width = (int)(width*YUVScaleX);
	height = (int)(height*YUVScaleY);

	if(!DisplayVideoOnly)
	{
		::ShowWindow(hTrack,SW_SHOW);
		::ShowWindow(hLeftButton,SW_SHOW);
		::ShowWindow(hLeftArrow,SW_SHOW);
		::ShowWindow(hRightArrow,SW_SHOW);
		::ShowWindow(hRightButton,SW_SHOW);
		MoveWindow(hTrack, 0, height, width-4*TRACK_HEIGHT, TRACK_HEIGHT, true);
		MoveWindow(hLeftButton, width-4*TRACK_HEIGHT, height, TRACK_HEIGHT, TRACK_HEIGHT, true);
		MoveWindow(hLeftArrow, width-3*TRACK_HEIGHT, height, TRACK_HEIGHT, TRACK_HEIGHT, true);
		MoveWindow(hRightArrow, width-2*TRACK_HEIGHT, height, TRACK_HEIGHT, TRACK_HEIGHT, true);
		MoveWindow(hRightButton, width-TRACK_HEIGHT, height, TRACK_HEIGHT, TRACK_HEIGHT, true);
	}
	else
	{
		::ShowWindow(hTrack,SW_HIDE);
		::ShowWindow(hLeftButton,SW_HIDE);
		::ShowWindow(hLeftArrow,SW_HIDE);
		::ShowWindow(hRightArrow,SW_HIDE);
		::ShowWindow(hRightButton,SW_HIDE);
	}
	GetWindowRect(hWnd, &wrect);
	GetClientRect(hWnd, &crect);
	Edge_Width = wrect.right - wrect.left - crect.right + crect.left;
	Edge_Height = wrect.bottom - wrect.top - crect.bottom + crect.top;
	
	if(DisplayVideoOnly)
		MoveWindow(hWnd, INIT_X, INIT_Y, width+Edge_Width, height+Edge_Height, true);
	else
		MoveWindow(hWnd, INIT_X, INIT_Y, width+Edge_Width, height+Edge_Height+TRACK_HEIGHT, true);

	// Now let's see if we need to notify our parent..
	if(Old_aspect_ratio == -1)
		Old_aspect_ratio = aspect_ratio_information;
	if(AutoScale && Old_aspect_ratio != aspect_ratio_information)
	{
		// DAR changed from under us! Let's send a message to the parent window.
		::SendMessage(ParenthWnd,WM_USER,ID_DAR_CHANGED,0);
	}
	Old_aspect_ratio = aspect_ratio_information;
}

void RefreshWindow(int update)
{
	if (update)
	{
		if (Check_Flag && WaitForSingleObject(hThread, 0)==WAIT_OBJECT_0)
		{
			Fault_Flag = false; 
			// JEANL I don't want this to be set automatically.
//			Display_Flag = true;
			Write_Frame(backward_reference_frame, d2v_backward, 0);
		}
	}
	else
		ShowFrame(true);
}

static void SaveBMP()
{
	// http://astronomy.swin.edu.au/~pbourke/dataformats/bmp/ for info on bmp format.
	unsigned char *dest = rgb24Scaled;
	unsigned char *src = rgb24;
	float FactX;
	float FactY;
	int TotalByteCount;
	FILE* fd ;
	int i,j;
	int NewWidth, NewHeight;
	int Clip_Width = Coded_Picture_Width;
	int Clip_Height = Coded_Picture_Height;
	BITMAPFILEHEADER BMPHeader = {'MB',0,0,0,sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER)};
	BITMAPINFOHEADER BMPInfoHeader;

	if(strcmp(BmpFileName,"") != 0)
	{
		strcpy(szTemp,BmpFileName);
	}
	else
	{
		if (!PopFileDlg(szTemp, hWnd, SAVE_BMP)) return;
		strcat(szTemp, ".bmp");
	}
	if ((fd = fopen(szTemp, "wb")) == NULL)
		return;

	// ARGHHHH learned the hard way! The width of a bmp picture HAS to be a multiple of 4!!!
	FactY = YDownSampFactor;
	FactX = XDownSampFactor;
	// We have to modify FactX so the resulting width will be a multiple of 4! BMP imposes that!!!
	NewWidth = 1+(int)((Clip_Width - 0.5)/FactX);
	NewWidth = 4*((int)((float)NewWidth/4+0.5));
	if(NewWidth == 0 || src == NULL)
	{
		// Image would become empty!
		return;
	}
	FactX = (float) Clip_Width / NewWidth;
	// Loop over lines:
	NewWidth=0;
	for(i=NewHeight=0,TotalByteCount=0;;i++)
	{
		// Find start of new line.
		int yindex = (int)(i * FactY + .5f);
		if(yindex >= Clip_Height)
			break;
		// The factor 3 comes from 3 bytes (R,B,G).
		src = rgb24 + yindex * Clip_Width * 3;
		// Copy this line.
		for(j=NewWidth=0;;j++)
		{
			// Find pixel location in line.
			int xindex = (int)(j * FactX + .5f);
			if(xindex >= Clip_Width)
				break;
			// Copy the three BGR components for that pixel.
			*dest++ = src[3*xindex];
			*dest++ = src[3*xindex+1];
			*dest++ = src[3*xindex+2];
			TotalByteCount+=3;
			NewWidth++;
		}
		NewHeight++;
	}

	// BMP (bitmap) file header stuff.
	// File header:
	BMPHeader.bfSize = sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER)+TotalByteCount;
	BMPInfoHeader.biSize = sizeof(BITMAPINFOHEADER);
	// Bitmap info header.
	BMPInfoHeader.biWidth = NewWidth;
	BMPInfoHeader.biHeight = NewHeight;
	BMPInfoHeader.biPlanes = 1;
	BMPInfoHeader.biBitCount = 24;
	BMPInfoHeader.biCompression = BI_RGB;
	BMPInfoHeader.biSizeImage = TotalByteCount;
	BMPInfoHeader.biXPelsPerMeter = 0;
	BMPInfoHeader.biYPelsPerMeter = 0;
	BMPInfoHeader.biClrUsed = 0;
	BMPInfoHeader.biClrImportant = 0;

	// Write headers.
	fwrite(&BMPHeader,sizeof(BITMAPFILEHEADER),1,fd);
	fwrite(&BMPInfoHeader,sizeof(BITMAPINFOHEADER),1,fd);
	// Then write data!
	fwrite(rgb24Scaled,sizeof(char),TotalByteCount,fd);
	fclose(fd);
}
/*
//jsoto:  commented lines

static void SaveBMP()
{
	FILE *BMPFile;

	if (!PopFileDlg(szTemp, hWnd, SAVE_BMP)) return;
	strcat(szTemp, ".bmp");
	if ((BMPFile = fopen(szTemp, "wb")) == NULL)
		return;

	int width = Coded_Picture_Width;
	int height = Coded_Picture_Height;

	if (ClipResize_Flag)
	{
		width -= Clip_Left+Clip_Right;
		height -= Clip_Top+Clip_Bottom;
	}

	BITMAPFILEHEADER bmfh;
	BITMAPINFOHEADER bmih;

	ZeroMemory(&bmfh, sizeof(BITMAPFILEHEADER));
	bmfh.bfType = 'M'*256 + 'B';
	bmfh.bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);
	bmfh.bfSize = bmfh.bfOffBits + ((width*24+31)&~31)/8*height;

	ZeroMemory(&bmih, sizeof(BITMAPINFOHEADER));
	bmih.biSize = sizeof(BITMAPINFOHEADER);
	bmih.biWidth = width;
	bmih.biHeight = height;
	bmih.biPlanes = 1;
	bmih.biBitCount = 24;
	bmih.biCompression = BI_RGB;
	bmih.biSizeImage = width * height * 3;

	fwrite(&bmfh, sizeof(BITMAPFILEHEADER), 1, BMPFile);
	fwrite(&bmih, sizeof(BITMAPINFOHEADER), 1, BMPFile);
	fwrite(rgb24, bmfh.bfSize - bmfh.bfOffBits, 1, BMPFile);

	fclose(BMPFile);
}
*/


static void StartupEnables(void)
{
	// This sets enabled and disabled controls for the startup condition.

/*
	// Main menu.
	EnableMenuItem(hMenu, 0, MF_BYPOSITION | MF_ENABLED);
	EnableMenuItem(hMenu, 1, MF_BYPOSITION | MF_ENABLED);
	EnableMenuItem(hMenu, 2, MF_BYPOSITION | MF_ENABLED);
	EnableMenuItem(hMenu, 3, MF_BYPOSITION | MF_ENABLED);
	EnableMenuItem(hMenu, 4, MF_BYPOSITION | MF_ENABLED);

	// File menu.
	EnableMenuItem(hMenu, IDM_OPEN, MF_ENABLED);
	EnableMenuItem(hMenu, IDM_OPEN_AUTO, MF_ENABLED);
	EnableMenuItem(hMenu, IDM_LOAD_D2V, MF_ENABLED);
	EnableMenuItem(hMenu, IDM_PROCESS_WAV, MF_ENABLED);
	EnableMenuItem(hMenu, IDM_SAVE_D2V, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_SAVE_D2V_176, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_SAVE, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_BMP, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_PLAY, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_PREVIEW, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_STOP, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_PAUSE, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_EXIT, MF_ENABLED);

	// Other menus are all enabled in the resource file.

	// Drag and drop.
	DragAcceptFiles(hWnd, true);
*/
	// Trackbar and buttons.
	EnableWindow(hTrack, false);
	EnableWindow(hLeftButton, false);
	EnableWindow(hLeftArrow, false);
	EnableWindow(hRightArrow, false);
	EnableWindow(hRightButton, false);

	// Refresh the menu bar.
//	DrawMenuBar(hWnd);
}

static void FileLoadedEnables(void)
{
	// Main menu.
/*	EnableMenuItem(hMenu, 0, MF_BYPOSITION | MF_ENABLED);
	EnableMenuItem(hMenu, 1, MF_BYPOSITION | MF_ENABLED);
	EnableMenuItem(hMenu, 2, MF_BYPOSITION | MF_ENABLED);
	EnableMenuItem(hMenu, 3, MF_BYPOSITION | MF_ENABLED);
	EnableMenuItem(hMenu, 4, MF_BYPOSITION | MF_ENABLED);

	// File menu.
	EnableMenuItem(hMenu, IDM_OPEN, MF_ENABLED);
	EnableMenuItem(hMenu, IDM_OPEN_AUTO, MF_ENABLED);
	EnableMenuItem(hMenu, IDM_LOAD_D2V, MF_ENABLED);
	EnableMenuItem(hMenu, IDM_PROCESS_WAV, MF_ENABLED);
	EnableMenuItem(hMenu, IDM_SAVE_D2V, MF_ENABLED);
	EnableMenuItem(hMenu, IDM_SAVE_D2V_176, MF_ENABLED);
	EnableMenuItem(hMenu, IDM_SAVE, MF_ENABLED);
	EnableMenuItem(hMenu, IDM_BMP, MF_ENABLED);
	EnableMenuItem(hMenu, IDM_PLAY, MF_ENABLED);
	EnableMenuItem(hMenu, IDM_PREVIEW, MF_ENABLED);
	EnableMenuItem(hMenu, IDM_STOP, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_PAUSE, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_EXIT, MF_ENABLED);

	// Drag and drop.
	DragAcceptFiles(hWnd, true);
*/
	// Trackbar and buttons.
	EnableWindow(hTrack, true);
	EnableWindow(hLeftButton, true);
	EnableWindow(hLeftArrow, true);
	EnableWindow(hRightArrow, true);
	EnableWindow(hRightButton, true);

	// Refresh the menu bar.
//	DrawMenuBar(hWnd);
}

static void RunningEnables(void)
{
	// Main menu.
/*	EnableMenuItem(hMenu, 0, MF_BYPOSITION | MF_ENABLED);
	EnableMenuItem(hMenu, 1, MF_BYPOSITION | MF_GRAYED);
	EnableMenuItem(hMenu, 2, MF_BYPOSITION | MF_GRAYED);
	EnableMenuItem(hMenu, 3, MF_BYPOSITION | MF_GRAYED);
	EnableMenuItem(hMenu, 4, MF_BYPOSITION | MF_GRAYED);

	// File menu.
	EnableMenuItem(hMenu, IDM_OPEN, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_OPEN_AUTO, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_LOAD_D2V, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_PROCESS_WAV, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_SAVE_D2V, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_SAVE_D2V_176, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_BMP, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_SAVE, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_PLAY, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_PREVIEW, MF_GRAYED);
	EnableMenuItem(hMenu, IDM_STOP, MF_ENABLED);
	EnableMenuItem(hMenu, IDM_PAUSE, MF_ENABLED);
	EnableMenuItem(hMenu, IDM_EXIT, MF_ENABLED);

	// Drag and drop.
	DragAcceptFiles(hWnd, false);
*/
	// Trackbar and buttons.
	EnableWindow(hTrack, false);
	EnableWindow(hLeftButton, false);
	EnableWindow(hLeftArrow, false);
	EnableWindow(hRightArrow, false);
	EnableWindow(hRightButton, false);

	// Refresh the menu bar.
//	DrawMenuBar(hWnd);
}
